# mode: run

# This test abuses the cname feature to modify the utility code so we can find out
# if the indexing functions are being called with unsafe_shared set.
# Then it checks if that matches our expectations.

# Don't worry about actually compiling with OpenMP though.
from cython.parallel cimport prange

# Intercept list indexing calls and record whether "unsafe_shared" was set
cdef extern from *:
    """
    static int unsafe_shared_set;

    static PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                int wraparound, int boundscheck, int unsafe_shared);
    static PyObject *Call_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                int wraparound, int boundscheck, int unsafe_shared) {
        int unique = (unsafe_shared == __Pyx_ReferenceSharing_DefinitelyUnique) || 
                     (unsafe_shared == __Pyx_ReferenceSharing_OwnStrongReference);
        unsafe_shared_set = !unique;
        return __Pyx_GetItemInt_List_Fast(o, i, wraparound, boundscheck, unsafe_shared);
    }

    #define do_nothing()
    """
    bint unsafe_shared_set
    void undef_getitemint_list_fast "#undef __Pyx_GetItemInt_List_Fast \n do_nothing" ()
    void define_new_getitemint_list_fast "#define __Pyx_GetItemInt_List_Fast(o, i, wraparound, boundscheck, unsafe_shared) Call_GetItemInt_List_Fast(o, i, wraparound, boundscheck, unsafe_shared) \n do_nothing" ()


def dummy_func():
    # abuse cname feature to redefine __Pyx_GetItemInt_List_Fast so that we can intercept the call
    define_new_getitemint_list_fast()

def local_var():
    """
    >>> local_var()
    False
    """
    global unsafe_shared_set
    unsafe_shared_set = False
    cdef list l = [None]
    x = l[0]
    return unsafe_shared_set

def closure1():
    """
    >>> closure1()
    True
    """
    global unsafe_shared_set
    unsafe_shared_set = False
    l = [None]

    def function_that_captures_l():
        return l
    x = l[0]
    return unsafe_shared_set

def closure2():
    """
    >>> closure2()
    True
    """
    global unsafe_shared_set
    unsafe_shared_set = False
    l = [None]

    def function_that_captures_l():
        return l[0]
    x = function_that_captures_l()
    return unsafe_shared_set

cdef class ListAttr:
    cdef list l
    def __init__(self):
        self.l = [None]

    def test_list_attr(self):
        """
        >>> ListAttr().test_list_attr()
        True
        """
        global unsafe_shared_set
        unsafe_shared_set = False
        x = self.l[0]
        return unsafe_shared_set


def in_prange():
    """
    >>> in_prange()
    True
    """
    global unsafe_shared_set
    unsafe_shared_set = False
    l = [None]
    for i in prange(1, nogil=True):
        with gil:
            x = l[0]
    return unsafe_shared_set


def dummy_func2():
    # abuse cname feature to undefine our override to allow the utility code to be generated
    undef_getitemint_list_fast()



# mode: run
# tag: c-api

from __future__ import print_function

from cpython cimport mem, PyObject
from cpython.pystate cimport PyGILState_Ensure, PyGILState_Release, PyGILState_STATE
from cpython.dict cimport PyDict_GetItemRef, PyDict_GetItemStringRef, PyDict_SetDefaultRef 
from cpython.list cimport PyList_GetItemRef, PyList_Clear, PyList_Extend
from cpython.ref cimport Py_XDECREF

import sys

__doc__ = ""

def test_pymalloc():
    """
    >>> test_pymalloc()
    3
    """
    cdef char* m2
    cdef char* m = <char*> mem.PyMem_Malloc(20)
    assert m
    try:
        m[0] = 1
        m[1] = 2
        m[2] = 3
        m2 = <char*> mem.PyMem_Realloc(m, 10)
        assert m2
        m = m2
        return m[2]
    finally:
        mem.PyMem_Free(m)


def test_gilstate():
    """
    >>> test_gilstate()
    'ok'
    """

    # cython used to have invalid definition for PyGILState_STATE, which was
    # making the following code fail to compile
    cdef PyGILState_STATE gstate = PyGILState_Ensure()
    # TODO assert that GIL is taken
    PyGILState_Release(gstate)
    return 'ok'

def test_dict_getref(d):
    """
    >>> test_dict_getref({'key': ['value']})
    ['value']
    ['value']
    1 ['value']
    >>> test_dict_getref({})
    0 Ellipsis

    One further test is in the module __doc__ to exclude it for PyPy
    """
    cdef PyObject* o;
    if PyDict_GetItemRef(d, "key", &o):
        print(<object>o)
        Py_XDECREF(o)
    
    if PyDict_GetItemStringRef(d, "key", &o):
        print(<object>o)
        Py_XDECREF(o)
    
    was_in_dict = PyDict_SetDefaultRef(d, "key", Ellipsis, &o)
    print(was_in_dict, <object>o)
    Py_XDECREF(o)

if not hasattr(sys, 'pypy_version_info'):
    __doc__ += """
        >>> test_dict_getref("I'm not a dict")  #doctest: +ELLIPSIS
        Traceback (most recent call last):
        SystemError: ...
    """

def test_list_getref_extend_clear(list):
    """
    >>> test_list_getref_extend_clear([1, 2, 3])
    1
    [1, 2, 3, 4, 5]
    []
    >>> test_list_getref_extend_clear([])  #doctest: +ELLIPSIS
    Traceback (most recent call last):
    IndexError: ...
    """

    first = PyList_GetItemRef(list, 0)
    print(first)

    PyList_Extend(list, [4, 5])
    print(list)
    
    PyList_Clear(list)
    print(list)



def cpdef_in_pxd(fail):
    """
    >>> cpdef_in_pxd(False)
    1
    >>> cpdef_in_pxd(True)
    Traceback (most recent call last):
    RuntimeError
    """
    if fail:
        raise RuntimeError()
    return 1



#!/usr/bin/python3

import json
import logging
import shutil
import datetime

from concurrent.futures import ProcessPoolExecutor as Pool, as_completed
from pathlib import Path
from urllib.request import urlopen, Request

logger = logging.getLogger()

PARALLEL_DOWNLOADS = 6
GITHUB_API_URL = "https://api.github.com/repos/cython/cython"


def find_github_files(version, api_url=GITHUB_API_URL):
    url = f"{api_url}/releases/tags/{version}"
    release, _ = read_url(url, accept="application/vnd.github+json", as_json=True)

    for asset in release.get('assets', ()):
        yield asset['browser_download_url']


def read_url(url, decode=True, accept=None, as_json=False):
    if accept:
        request = Request(url, headers={'Accept': accept})
    else:
        request = Request(url)

    with urlopen(request) as res:
        charset = _find_content_encoding(res)
        content_type = res.headers.get('Content-Type')
        data = res.read()

    if decode:
        data = data.decode(charset)
    if as_json:
        data = json.loads(data)
    return data, content_type


def _find_content_encoding(response, default='iso8859-1'):
    from email.message import Message
    content_type = response.headers.get('Content-Type')
    if content_type:
        msg = Message()
        msg.add_header('Content-Type', content_type)
        charset = msg.get_content_charset(default)
    else:
        charset = default
    return charset


def download1(wheel_url, dest_dir):
    wheel_name = wheel_url.rsplit("/", 1)[1]
    logger.info(f"Downloading {wheel_url} ...")
    with urlopen(wheel_url) as w:
        file_path = dest_dir / wheel_name
        if (file_path.exists()
                and "Content-Length" in w.headers
                and file_path.stat().st_size == int(w.headers["Content-Length"])):
            logger.info(f"Already have {wheel_name}")
        else:
            temp_file_path = file_path.with_suffix(".tmp")
            try:
                with open(temp_file_path, "wb") as f:
                    shutil.copyfileobj(w, f)
            except:
                if temp_file_path.exists():
                    temp_file_path.unlink()
                raise
            else:
                temp_file_path.replace(file_path)
                logger.info(f"Finished downloading {wheel_name}")
    return wheel_name


def download(urls, dest_dir, jobs=PARALLEL_DOWNLOADS):
    with Pool(max_workers=jobs) as pool:
        futures = [pool.submit(download1, url, dest_dir) for url in urls]
        try:
            for future in as_completed(futures):
                wheel_name = future.result()
                yield wheel_name
        except KeyboardInterrupt:
            for future in futures:
                future.cancel()
            raise


def dedup(it):
    seen = set()
    for value in it:
        if value not in seen:
            seen.add(value)
            yield value


def roundrobin(*iterables):
    "roundrobin('ABC', 'D', 'EF') --> A D E B F C"
    # Recipe credited to George Sakkis
    from itertools import cycle, islice
    num_active = len(iterables)
    nexts = cycle(iter(it).__next__ for it in iterables)
    while num_active:
        try:
            for next in nexts:
                yield next()
        except StopIteration:
            # Remove the iterator we just exhausted from the cycle.
            num_active -= 1
            nexts = cycle(islice(nexts, num_active))


def main(*args):
    if not args:
        print("Please pass the version to download")
        return

    version = args[0]
    dest_dir = Path("dist") / version
    if not dest_dir.is_dir():
        dest_dir.mkdir()

    start_time = datetime.datetime.now().replace(microsecond=0)
    urls = roundrobin(*map(dedup, [
        find_github_files(version),
    ]))
    count = sum(1 for _ in download(urls, dest_dir))
    duration = datetime.datetime.now().replace(microsecond=0) - start_time
    logger.info(f"Downloaded {count} files in {duration}.")


if __name__ == "__main__":
    import sys
    logging.basicConfig(
        stream=sys.stderr,
        level=logging.INFO,
        format="%(asctime)-15s  %(message)s",
    )
    main(*sys.argv[1:])



# mode: compile
# tag: cpp, cpp17, no-cpp-locals

# This test tests a small optimization for except clauses where they can
# potentially omit some exception handling code if sufficiently simple.
# The optimization is performed explicitly because it's completely
# unobservable. Therefore, it's hard to test behaviourally.
#
# Restrict to c++17 just because we use cpp_locals in the test.

cimport cython

cdef cppclass CppClass:
    int attr
    CppClass():
        pass

def return_empty():
    with cython.test_body_needs_exception_handling(False):
        return

def return_none():
    with cython.test_body_needs_exception_handling(False):
        return None

def return_int():
    with cython.test_body_needs_exception_handling(False):
        return 100

def return_str_literal():
    with cython.test_body_needs_exception_handling(False):
        return "I'm always in the module cache!"

def return_arg(arg):
    with cython.test_body_needs_exception_handling(False):
        return arg

def return_assigned():
    x = object()
    with cython.test_body_needs_exception_handling(False):
        return x

def result_maybe_unassigned(arg):
    if arg:
        x = object
    with cython.test_body_needs_exception_handling(True):
        return x

py_global = None
cdef int c_global = 0

def return_py_global():
    with cython.test_body_needs_exception_handling(True):
        return py_global

def return_c_global():
    with cython.test_body_needs_exception_handling(True):
        return c_global

def test_pass():
    with cython.test_body_needs_exception_handling(False):
        pass

cdef void noexcept_func() noexcept:
    pass

def call_noexcept_func():
    with cython.test_body_needs_exception_handling(True):
        noexcept_func()

def test_assignment(arg):
    global py_global, c_global

    cdef str typed_as_string = "hello"

    cdef list this_is_a_list = []

    some_value = 5
    with cython.test_body_needs_exception_handling(False):
        a = 2
        b = None
        c = some_value
        d = arg
        typed_as_string = "goodbye"  # string destructor is safe

    if arg:
        some_other_value = object()
    cdef float some_c_value = 0.0
    with cython.test_body_needs_exception_handling(True):
        e = some_other_value
    with cython.test_body_needs_exception_handling(False):
        # Re-assignment of Python or C objects isn't a problem.
        # We're treating it as undefined if the destructor gets called
        # during the exception handler.
        some_other_value = None  # may call destructor
        some_c_value = 1.0  # no destructor
    with cython.test_body_needs_exception_handling(True):
        this_is_a_list = arg  # type-check
    with cython.test_body_needs_exception_handling(True):
        py_global = True
    with cython.test_body_needs_exception_handling(False):
        c_global = 1

    cdef CppClass cpp_instance1
    cdef CppClass cpp_instance2

    with cython.test_body_needs_exception_handling(True):
        cpp_instance1 = cpp_instance2  # operator= can be non-trivial

def test_memoryview_assignment(obj):
    cdef int[:] obj_view = obj
    cdef int[:] uninitialized_view
    cdef int[:] a, b, c

    with cython.test_body_needs_exception_handling(True):
        a = obj  # this can fail
    
    with cython.test_body_needs_exception_handling(False):
        b = obj_view
        # `a` may call destructor on re-assignment but we're
        # treating that as implementation-defined.
        a = obj_view

cdef class CClass:
    cdef int x
    cdef object y
    cdef int[:] z

def test_attr_assignment(CClass arg):
    cdef int[:] mview = arg
    cdef int[:] unassigned_mview
    o = object()
    with cython.test_body_needs_exception_handling(False):
        arg.x = 5
        something = arg.x
        arg.y = o
        arg.z = mview
    with cython.test_body_needs_exception_handling(True):
        unassigned_mview = arg.z  # needs initialized check
    with cython.test_body_needs_exception_handling(True):
        arg.unknown_attr = 1
    with cython.test_body_needs_exception_handling(True):
        b = arg.unknown_attr

def test_maybe_unassigned_int(arg):
    cdef int a, b
    if arg:
        maybe_unassigned = 5
    with cython.test_body_needs_exception_handling(False):
        b = maybe_unassigned

@cython.cpp_locals(True)
def test_cpp_locals(arg):
    # Currently, cpp classes are need exception handling because
    # they may have a non-trivial assignment operator. So by
    # definition cpp_locals also need exception handling.
    if arg:
        maybe_unassigned = CppClass()
    with cython.test_body_needs_exception_handling(True):
        maybe_unassigned.attr = 1
    with cython.test_body_needs_exception_handling(True):
        a = arg.maybe_unassigned

def test_indexing(s):
    cdef list this_is_a_list = []
    cdef const char *s_ptr = s
    cdef int[5] arr
    with cython.test_body_needs_exception_handling(True):
        this_is_a_list[0]
    with cython.test_body_needs_exception_handling(False):
        s_ptr[0]
        arr[0]



@cython.cclass
class Shrubbery:
    width: cython.int
    height: cython.int

    def __init__(self, w, h):
        self.width = w
        self.height = h

    def describe(self):
        print("This shrubbery is", self.width,
              "by", self.height, "cubits.")



# mode: run
# tag: c11, no-cpp, no-macos

# cython: language_level=3

from libc cimport threads
from libc cimport time as ctime

from threading import Thread, Barrier
import time

def test_mutex():
    """
    test_mutex()
    """
    cdef threads.mtx_t m
    threads.mtx_init(&m, threads.mtx_plain)
    threads.mtx_lock(&m)
    threads.mtx_unlock(&m)
    threads.mtx_destroy(&m)

cdef void call_me_once() noexcept with gil:
    # with gil is only OK because it's a toy example with no other threads so no chance of deadlock.
    # Do not copy this code!
    print("Listen very carefully, I shall say this only once.")

def test_once():
    """
    >>> test_once()
    Listen very carefully, I shall say this only once.
    """
    cdef threads.once_flag flag
    threads.once_flag_init(&flag)
    threads.call_once(&flag, &call_me_once)
    threads.call_once(&flag, &call_me_once)

cdef int my_thread_func(void* arg) noexcept nogil:
    cdef int x = (<int*>arg)[0]
    t = threads.thrd_current()  # compile test - nothing useful to do with it
    return x

def test_thread():
    """
    >>> test_thread()
    """
    cdef threads.thrd_t t
    cdef int one = 1
    threads.thrd_create(&t, my_thread_func, &one)
    cdef int result = 0
    threads.thrd_join(t, &result)
    assert result==1, result

def test_py_safe_lock(n_threads):
    """
    >>> test_py_safe_lock(4)
    2000
    """
    cdef threads.mtx_t m
    threads.mtx_init(&m, threads.mtx_plain)

    barrier = Barrier(n_threads)
    cdef int count = 0

    def thread_func():
        nonlocal count

        barrier.wait()
        for i in range(500):
            if i%2:
                with nogil:
                    threads.py_safe_mtx_lock(&m)
                    try:
                        count += 1
                    finally:
                        threads.mtx_unlock(&m)
            else:
                threads.py_safe_mtx_lock(&m)
                try:
                    count += 1
                finally:
                    threads.mtx_unlock(&m)

    all_threads = [
        Thread(target=thread_func) for _ in range(n_threads)
    ]
    for t in all_threads:
        t.start()
    for t in all_threads:
        t.join()
    print(count)
    threads.mtx_destroy(&m)

def test_py_safe_lock_nogil():
    """
    >>> test_py_safe_lock_nogil()
    """
    cdef threads.mtx_t m
    threads.mtx_init(&m, threads.mtx_plain)

    with nogil:
        threads.py_safe_mtx_lock(&m)
        threads.mtx_unlock(&m)

    threads.mtx_destroy(&m)

def test_py_safe_cnd_wait():
    """
    >>> test_py_safe_cnd_wait()
    10 True
    """
    cdef threads.cnd_t condition_var
    threads.cnd_init(&condition_var)
    cdef threads.mtx_t m
    threads.mtx_init(&m, threads.mtx_plain)

    cdef object value = 0
    cdef object done = False

    def wait_on_condition_var():
        nonlocal done

        threads.py_safe_mtx_lock(&m)
        try:
            wait_result = threads.thrd_success
            while wait_result == threads.thrd_success and value <= 5:
                threads.py_safe_cnd_wait(&condition_var, &m)
            if value > 5:
                done = True
        finally:
            threads.mtx_unlock(&m)

    t1 = Thread(target=wait_on_condition_var)
    t1.start()

    def set_value():
        nonlocal value
        for i in range(10):
            threads.py_safe_mtx_lock(&m)
            try:
                value += 1
            finally:
                threads.mtx_unlock(&m)
            threads.cnd_signal(&condition_var)

    t2 = Thread(target=set_value)
    t2.start()

    t1.join()
    t2.join()
    threads.mtx_destroy(&m)
    threads.cnd_destroy(&condition_var)

    print(value, done)

def test_py_safe_cnd_timed_wait(fail):
    """
    >>> test_py_safe_cnd_timed_wait(False)
    True
    >>> test_py_safe_cnd_timed_wait(True)
    False
    """
    cdef threads.cnd_t condition_var
    threads.cnd_init(&condition_var)
    cdef threads.mtx_t m
    threads.mtx_init(&m, threads.mtx_plain)

    condition = False
    condition_var_succeeded = False

    def wait_on_condition_var():
        nonlocal condition_var_succeeded

        cdef ctime.timespec ts
        ctime.timespec_get(&ts, ctime.TIME_UTC);
        if fail:
            # don't wait too long to not do anything
            ts.tv_nsec += 100_000_000  # 100 ms
            if ts.tv_nsec > 1000_000_000:  # 1s
                ts.tv_nsec -= 1000_000_000
                ts.tv_sec += 1
        else:
            ts.tv_sec += 10  # make the test robust

        threads.py_safe_mtx_lock(&m)
        try:
            result = threads.thrd_success
            while result == threads.thrd_success and not condition:
                result = threads.py_safe_cnd_timedwait(
                    &condition_var, &m, &ts
                )
            if condition:
                condition_var_succeeded = True
                return
        finally:
            threads.mtx_unlock(&m)

    t = Thread(target=wait_on_condition_var)
    t.start()
    threads.py_safe_mtx_lock(&m)
    try:
        if not fail:
            condition = True
            threads.cnd_broadcast(&condition_var)
    finally:
        threads.mtx_unlock(&m);

    t.join()
    print(condition_var_succeeded)
    threads.mtx_destroy(&m)
    threads.cnd_destroy(&condition_var)



# mode: error

# This is split into many sub-files because some errors terminate the parser

t'}'
t'{}'
t'{=x}'

_ERRORS = """
5:2: f-string or t-string: single '}' is not allowed
6:2: empty expression not allowed in t-string
7:3: Expected an identifier or literal
"""



import cython

import collections
import time


builtin_collections = cython.fused_type(
    list,
    tuple,
    set,
    dict,
    object,
)

# FIXME: This should work but currently fails to define a fused type alias name:
# builtin_collections2 = cython.typedef(builtin_collections)
# builtin_collections3 = cython.typedef(builtin_collections)

builtin_collections2 = cython.fused_type(
    list,
    tuple,
    set,
    dict,
    object,
)

builtin_collections3 = cython.fused_type(
    list,
    tuple,
    set,
    dict,
    object,
)


def _fused_func_args_1(o: builtin_collections):
    assert o is not None


def _call_fused_func_args_1(ordered: cython.bint, number: cython.int, timer):
    func = _fused_func_args_1
    args = [[], (), set(), {}, 5]
    number //= len(args)

    t = timer()
    if ordered:
        for arg in args:
            for _ in range(number):
                func(arg)
    else:
        for _ in range(number):
            for arg in args:
                func(arg)
    t = timer() - t
    return t


def _fused_func_args_2(o1: builtin_collections, o2: builtin_collections2):
    assert o1 is not None
    assert o2 is not None


def _call_fused_func_args_2(ordered: cython.bint, number: cython.int, timer):
    func = _fused_func_args_2
    args = [[], (), set(), {}, 5]
    number //= len(args) ** 2

    t = timer()
    if ordered:
        for arg1 in args:
            for arg2 in args:
                for _ in range(number):
                    func(arg1, arg2)
    else:
        for _ in range(number):
            for arg1 in args:
                for arg2 in args:
                    func(arg1, arg2)
    t = timer() - t
    return t


def _fused_func_args_3(o1: builtin_collections, o2: builtin_collections2, o3: builtin_collections3):
    assert o1 is not None
    assert o2 is not None
    assert o3 is not None


def _call_fused_func_args_3(ordered: cython.bint, number: cython.int, timer):
    func = _fused_func_args_3
    args = [[], (), set(), {}, 5]
    number //= len(args) ** 3

    t = timer()
    if ordered:
        for arg1 in args:
            for arg2 in args:
                for arg3 in args:
                    for _ in range(number):
                        func(arg1, arg2, arg3)
    else:
        for _ in range(number):
            for arg1 in args:
                for arg2 in args:
                    for arg3 in args:
                        func(arg1, arg2, arg3)
    t = timer() - t
    return t


def bm_fused_args(number, timer=time.perf_counter):
    return {
        'fused_args_1_ordered': _call_fused_func_args_1(True, number, timer),
        'fused_args_1_unordered': _call_fused_func_args_1(False, number, timer),
        'fused_args_2_ordered': _call_fused_func_args_2(True, number, timer),
        'fused_args_2_unordered': _call_fused_func_args_2(False, number, timer),
        'fused_args_3_ordered': _call_fused_func_args_3(True, number, timer),
        'fused_args_3_unordered': _call_fused_func_args_3(False, number, timer),
    }


def run_benchmark(repeat: cython.int = 10, number=100, timer=time.perf_counter):
    i: cython.int

    collected_timings = collections.defaultdict(list)

    for name, func in globals().items():
        if not name.startswith('bm_'):
            continue

        for i in range(repeat):
            timings = func(number, timer)
            for name, t in timings.items():
                collected_timings[name].append(t)

    for name, timings in collected_timings.items():
        print(f"{name}: {timings}")



# mode: run
# tag: cpp, cpp11, no-cpp-locals

# cython: freethreading_compatible=True

# List indexing should be thread-safe (at least as far as not generating
# reference counting errors).
# It's actually surprisingly hard to make a test that reliably breaks and
# so this is pretty tailored to CPython's implementation.
# Essentially you want to catch it in "quick" deallocation (where objects
# are created and destroyed by the same thread).

cimport cython

from libcpp.atomic cimport atomic
from threading import Thread, Barrier

def run(reader, writer, n_threads, n_loops, *args):
    """
    >>> import gc
    >>> import sys
    >>> last_count = Counted.get_count()
    >>> run(read_local_var, write_local_var, 2, 2000, [None])
    >>> _ = gc.collect()
    >>> Counted.get_count() - last_count
    0

    >>> last_count = Counted.get_count()
    >>> run(read_global_var, write_global_var, 4, 100)
    >>> _ = gc.collect()
    >>> Counted.get_count() - last_count
    0

    >>> last_count = Counted.get_count()
    >>> closure_reader, closure_writer = make_closure_funcs()
    >>> run(closure_reader, closure_writer, 4, 100)
    >>> _ = gc.collect()
    >>> Counted.get_count() - last_count
    0
    """
    barrier = Barrier(n_threads+1)
    def call_read():
        barrier.wait()
        reader(n_loops, *args)

    def call_write(arg):
        barrier.wait()
        writer(n_loops, *args)

    l = [None]
    threads = [ Thread(target=call_read) for _ in range(n_threads) ]
    for t in threads:
        t.start()
    call_write(l)
    for t in threads:
        t.join()

cdef atomic[int] count
count.store(0)

cdef class Counted:
    def __cinit__(self):
        count.fetch_add(1)

    def __dealloc__(self):
        count.fetch_sub(1)

    @staticmethod
    def get_count():
        return count.load()

def read_local_var(int n_loops, list l not None):
    for _ in range(n_loops):
        tmp = l[0]

def write_local_var(int n_loops, list l not None):
    for _ in range(n_loops):
        l[0] = Counted()

cdef list global_list = [None]

def read_global_var(int n_loops):
    for _ in range(n_loops):
        tmp = global_list[0]

def write_global_var(int n_loops):
    for _ in range(n_loops):
        global_list[0] = Counted()
    global_list[0] = None


def make_closure_funcs():
    l = [None]
    def reader(int n_loops):
        for n in range(n_loops):
            tmp = l[0]
    def writer(int n_loops):
        for n in range(n_loops):
            l[0] = Counted()
        l[0] = None
    return reader, writer



# cython: language_level=3
# mode: run
# tag: allow_unknown_names, f_strings, pep498

import ast
import os
import types
import decimal
import unittest
import re
import contextlib
import sys

from Cython.Build.Inline import cython_inline
from Cython.TestUtils import CythonTest
from Cython.Compiler.Errors import CompileError, hold_errors, init_thread, held_errors

def cy_eval(s, **kwargs):
    return cython_inline('return ' + s, force=True, **kwargs)

a_global = 'global variable'

# You could argue that I'm too strict in looking for specific error
#  values with assertRaisesRegex, but without it it's way too easy to
#  make a syntax error in the test strings. Especially with all of the
#  triple quotes, raw strings, backslashes, etc. I think it's a
#  worthwhile tradeoff. When I switched to this method, I found many
#  examples where I wasn't testing what I thought I was.

class TestCase(CythonTest):
    def assertAllRaise(self, exception_type, regex, error_strings):
        for str in error_strings:
            hold_errors()
            if exception_type is SyntaxError:
                try:
                    self.fragment(str)
                except CompileError:
                    assert True
                else:
                    assert held_errors(), "Invalid Cython code failed to raise SyntaxError: %r" % str
                finally:
                    init_thread()  # reset error status
            else:
                try:
                    cython_inline(str, quiet=True)
                except exception_type:
                    assert True
                else:
                    assert False, "Invalid Cython code failed to raise %s: %r" % (exception_type, str)
                finally:
                    init_thread()  # reset error status

    @contextlib.contextmanager
    def assertRaisesRegex(self, exception, regex, *, msg=None):
        # We can't hope to match CPython's exact syntax errors
        hold_errors()
        try:
            if exception is SyntaxError:
                with self.assertRaises(exception, msg=msg):
                    try:
                        yield
                    except CompileError as e:
                        raise SyntaxError(e.args)
                    else:
                        if held_errors():
                            raise SyntaxError()
            else:
                with super().assertRaisesRegex(exception, regex, msg=msg):
                    yield
        finally:
            init_thread()  # reset error status

    def test__format__lookup(self):
        # Make sure __format__ is looked up on the type, not the instance.
        class X:
            def __format__(self, spec):
                return 'class'

        x = X()

        # Add a bound __format__ method to the 'y' instance, but not
        #  the 'x' instance.
        y = X()
        y.__format__ = types.MethodType(lambda self, spec: 'instance', y)

        self.assertEqual(f'{y}', format(y))
        self.assertEqual(f'{y}', 'class')
        self.assertEqual(format(x), format(y))

        # __format__ is not called this way, but still make sure it
        #  returns what we expect (so we can make sure we're bypassing
        #  it).
        self.assertEqual(x.__format__(''), 'class')
        self.assertEqual(y.__format__(''), 'instance')

        # This is how __format__ is actually called.
        self.assertEqual(type(x).__format__(x, ''), 'class')
        self.assertEqual(type(y).__format__(y, ''), 'class')

    def __test_ast(self):
        # Inspired by https://bugs.python.org/issue24975
        class X:
            def __init__(self):
                self.called = False
            def __call__(self):
                self.called = True
                return 4
        x = X()
        expr = """
a = 10
f'{a * x()}'"""
        t = ast.parse(expr)
        c = compile(t, '', 'exec')

        # Make sure x was not called.
        self.assertFalse(x.called)

        # Actually run the code.
        exec(c)

        # Make sure x was called.
        self.assertTrue(x.called)

    def __test_ast_line_numbers(self):
        expr = """
a = 10
f'{a * x()}'"""
        t = ast.parse(expr)
        self.assertEqual(type(t), ast.Module)
        self.assertEqual(len(t.body), 2)
        # check `a = 10`
        self.assertEqual(type(t.body[0]), ast.Assign)
        self.assertEqual(t.body[0].lineno, 2)
        # check `f'...'`
        self.assertEqual(type(t.body[1]), ast.Expr)
        self.assertEqual(type(t.body[1].value), ast.JoinedStr)
        self.assertEqual(len(t.body[1].value.values), 1)
        self.assertEqual(type(t.body[1].value.values[0]), ast.FormattedValue)
        self.assertEqual(t.body[1].lineno, 3)
        self.assertEqual(t.body[1].value.lineno, 3)
        self.assertEqual(t.body[1].value.values[0].lineno, 3)
        # check the binop location
        binop = t.body[1].value.values[0].value
        self.assertEqual(type(binop), ast.BinOp)
        self.assertEqual(type(binop.left), ast.Name)
        self.assertEqual(type(binop.op), ast.Mult)
        self.assertEqual(type(binop.right), ast.Call)
        self.assertEqual(binop.lineno, 3)
        self.assertEqual(binop.left.lineno, 3)
        self.assertEqual(binop.right.lineno, 3)
        self.assertEqual(binop.col_offset, 3)
        self.assertEqual(binop.left.col_offset, 3)
        self.assertEqual(binop.right.col_offset, 7)

    def __test_ast_line_numbers_multiple_formattedvalues(self):
        expr = """
f'no formatted values'
f'eggs {a * x()} spam {b + y()}'"""
        t = ast.parse(expr)
        self.assertEqual(type(t), ast.Module)
        self.assertEqual(len(t.body), 2)
        # check `f'no formatted value'`
        self.assertEqual(type(t.body[0]), ast.Expr)
        self.assertEqual(type(t.body[0].value), ast.JoinedStr)
        self.assertEqual(t.body[0].lineno, 2)
        # check `f'...'`
        self.assertEqual(type(t.body[1]), ast.Expr)
        self.assertEqual(type(t.body[1].value), ast.JoinedStr)
        self.assertEqual(len(t.body[1].value.values), 4)
        self.assertEqual(type(t.body[1].value.values[0]), ast.Constant)
        self.assertEqual(type(t.body[1].value.values[0].value), str)
        self.assertEqual(type(t.body[1].value.values[1]), ast.FormattedValue)
        self.assertEqual(type(t.body[1].value.values[2]), ast.Constant)
        self.assertEqual(type(t.body[1].value.values[2].value), str)
        self.assertEqual(type(t.body[1].value.values[3]), ast.FormattedValue)
        self.assertEqual(t.body[1].lineno, 3)
        self.assertEqual(t.body[1].value.lineno, 3)
        self.assertEqual(t.body[1].value.values[0].lineno, 3)
        self.assertEqual(t.body[1].value.values[1].lineno, 3)
        self.assertEqual(t.body[1].value.values[2].lineno, 3)
        self.assertEqual(t.body[1].value.values[3].lineno, 3)
        # check the first binop location
        binop1 = t.body[1].value.values[1].value
        self.assertEqual(type(binop1), ast.BinOp)
        self.assertEqual(type(binop1.left), ast.Name)
        self.assertEqual(type(binop1.op), ast.Mult)
        self.assertEqual(type(binop1.right), ast.Call)
        self.assertEqual(binop1.lineno, 3)
        self.assertEqual(binop1.left.lineno, 3)
        self.assertEqual(binop1.right.lineno, 3)
        self.assertEqual(binop1.col_offset, 8)
        self.assertEqual(binop1.left.col_offset, 8)
        self.assertEqual(binop1.right.col_offset, 12)
        # check the second binop location
        binop2 = t.body[1].value.values[3].value
        self.assertEqual(type(binop2), ast.BinOp)
        self.assertEqual(type(binop2.left), ast.Name)
        self.assertEqual(type(binop2.op), ast.Add)
        self.assertEqual(type(binop2.right), ast.Call)
        self.assertEqual(binop2.lineno, 3)
        self.assertEqual(binop2.left.lineno, 3)
        self.assertEqual(binop2.right.lineno, 3)
        self.assertEqual(binop2.col_offset, 23)
        self.assertEqual(binop2.left.col_offset, 23)
        self.assertEqual(binop2.right.col_offset, 27)

    def __test_ast_line_numbers_nested(self):
        expr = """
a = 10
f'{a * f"-{x()}-"}'"""
        t = ast.parse(expr)
        self.assertEqual(type(t), ast.Module)
        self.assertEqual(len(t.body), 2)
        # check `a = 10`
        self.assertEqual(type(t.body[0]), ast.Assign)
        self.assertEqual(t.body[0].lineno, 2)
        # check `f'...'`
        self.assertEqual(type(t.body[1]), ast.Expr)
        self.assertEqual(type(t.body[1].value), ast.JoinedStr)
        self.assertEqual(len(t.body[1].value.values), 1)
        self.assertEqual(type(t.body[1].value.values[0]), ast.FormattedValue)
        self.assertEqual(t.body[1].lineno, 3)
        self.assertEqual(t.body[1].value.lineno, 3)
        self.assertEqual(t.body[1].value.values[0].lineno, 3)
        # check the binop location
        binop = t.body[1].value.values[0].value
        self.assertEqual(type(binop), ast.BinOp)
        self.assertEqual(type(binop.left), ast.Name)
        self.assertEqual(type(binop.op), ast.Mult)
        self.assertEqual(type(binop.right), ast.JoinedStr)
        self.assertEqual(binop.lineno, 3)
        self.assertEqual(binop.left.lineno, 3)
        self.assertEqual(binop.right.lineno, 3)
        self.assertEqual(binop.col_offset, 3)
        self.assertEqual(binop.left.col_offset, 3)
        self.assertEqual(binop.right.col_offset, 7)
        # check the nested call location
        self.assertEqual(len(binop.right.values), 3)
        self.assertEqual(type(binop.right.values[0]), ast.Constant)
        self.assertEqual(type(binop.right.values[0].value), str)
        self.assertEqual(type(binop.right.values[1]), ast.FormattedValue)
        self.assertEqual(type(binop.right.values[2]), ast.Constant)
        self.assertEqual(type(binop.right.values[2].value), str)
        self.assertEqual(binop.right.values[0].lineno, 3)
        self.assertEqual(binop.right.values[1].lineno, 3)
        self.assertEqual(binop.right.values[2].lineno, 3)
        call = binop.right.values[1].value
        self.assertEqual(type(call), ast.Call)
        self.assertEqual(call.lineno, 3)
        self.assertEqual(call.col_offset, 11)

    def __test_ast_line_numbers_duplicate_expression(self):
        """Duplicate expression

        NOTE: this is currently broken, always sets location of the first
        expression.
        """
        expr = """
a = 10
f'{a * x()} {a * x()} {a * x()}'
"""
        t = ast.parse(expr)
        self.assertEqual(type(t), ast.Module)
        self.assertEqual(len(t.body), 2)
        # check `a = 10`
        self.assertEqual(type(t.body[0]), ast.Assign)
        self.assertEqual(t.body[0].lineno, 2)
        # check `f'...'`
        self.assertEqual(type(t.body[1]), ast.Expr)
        self.assertEqual(type(t.body[1].value), ast.JoinedStr)
        self.assertEqual(len(t.body[1].value.values), 5)
        self.assertEqual(type(t.body[1].value.values[0]), ast.FormattedValue)
        self.assertEqual(type(t.body[1].value.values[1]), ast.Constant)
        self.assertEqual(type(t.body[1].value.values[1].value), str)
        self.assertEqual(type(t.body[1].value.values[2]), ast.FormattedValue)
        self.assertEqual(type(t.body[1].value.values[3]), ast.Constant)
        self.assertEqual(type(t.body[1].value.values[3].value), str)
        self.assertEqual(type(t.body[1].value.values[4]), ast.FormattedValue)
        self.assertEqual(t.body[1].lineno, 3)
        self.assertEqual(t.body[1].value.lineno, 3)
        self.assertEqual(t.body[1].value.values[0].lineno, 3)
        self.assertEqual(t.body[1].value.values[1].lineno, 3)
        self.assertEqual(t.body[1].value.values[2].lineno, 3)
        self.assertEqual(t.body[1].value.values[3].lineno, 3)
        self.assertEqual(t.body[1].value.values[4].lineno, 3)
        # check the first binop location
        binop = t.body[1].value.values[0].value
        self.assertEqual(type(binop), ast.BinOp)
        self.assertEqual(type(binop.left), ast.Name)
        self.assertEqual(type(binop.op), ast.Mult)
        self.assertEqual(type(binop.right), ast.Call)
        self.assertEqual(binop.lineno, 3)
        self.assertEqual(binop.left.lineno, 3)
        self.assertEqual(binop.right.lineno, 3)
        self.assertEqual(binop.col_offset, 3)
        self.assertEqual(binop.left.col_offset, 3)
        self.assertEqual(binop.right.col_offset, 7)
        # check the second binop location
        binop = t.body[1].value.values[2].value
        self.assertEqual(type(binop), ast.BinOp)
        self.assertEqual(type(binop.left), ast.Name)
        self.assertEqual(type(binop.op), ast.Mult)
        self.assertEqual(type(binop.right), ast.Call)
        self.assertEqual(binop.lineno, 3)
        self.assertEqual(binop.left.lineno, 3)
        self.assertEqual(binop.right.lineno, 3)
        self.assertEqual(binop.col_offset, 13)
        self.assertEqual(binop.left.col_offset, 13)
        self.assertEqual(binop.right.col_offset, 17)
        # check the third binop location
        binop = t.body[1].value.values[4].value
        self.assertEqual(type(binop), ast.BinOp)
        self.assertEqual(type(binop.left), ast.Name)
        self.assertEqual(type(binop.op), ast.Mult)
        self.assertEqual(type(binop.right), ast.Call)
        self.assertEqual(binop.lineno, 3)
        self.assertEqual(binop.left.lineno, 3)
        self.assertEqual(binop.right.lineno, 3)
        self.assertEqual(binop.col_offset, 23)
        self.assertEqual(binop.left.col_offset, 23)
        self.assertEqual(binop.right.col_offset, 27)

    def __test_ast_numbers_fstring_with_formatting(self):

        t = ast.parse('f"Here is that pesky {xxx:.3f} again"')
        self.assertEqual(len(t.body), 1)
        self.assertEqual(t.body[0].lineno, 1)

        self.assertEqual(type(t.body[0]), ast.Expr)
        self.assertEqual(type(t.body[0].value), ast.JoinedStr)
        self.assertEqual(len(t.body[0].value.values), 3)

        self.assertEqual(type(t.body[0].value.values[0]), ast.Constant)
        self.assertEqual(type(t.body[0].value.values[1]), ast.FormattedValue)
        self.assertEqual(type(t.body[0].value.values[2]), ast.Constant)

        _, expr, _ = t.body[0].value.values

        name = expr.value
        self.assertEqual(type(name), ast.Name)
        self.assertEqual(name.lineno, 1)
        self.assertEqual(name.end_lineno, 1)
        self.assertEqual(name.col_offset, 22)
        self.assertEqual(name.end_col_offset, 25)

    def __test_ast_line_numbers_multiline_fstring(self):
        # See bpo-30465 for details.
        expr = """
a = 10
f'''
  {a
     *
       x()}
non-important content
'''
"""
        t = ast.parse(expr)
        self.assertEqual(type(t), ast.Module)
        self.assertEqual(len(t.body), 2)
        # check `a = 10`
        self.assertEqual(type(t.body[0]), ast.Assign)
        self.assertEqual(t.body[0].lineno, 2)
        # check `f'...'`
        self.assertEqual(type(t.body[1]), ast.Expr)
        self.assertEqual(type(t.body[1].value), ast.JoinedStr)
        self.assertEqual(len(t.body[1].value.values), 3)
        self.assertEqual(type(t.body[1].value.values[0]), ast.Constant)
        self.assertEqual(type(t.body[1].value.values[0].value), str)
        self.assertEqual(type(t.body[1].value.values[1]), ast.FormattedValue)
        self.assertEqual(type(t.body[1].value.values[2]), ast.Constant)
        self.assertEqual(type(t.body[1].value.values[2].value), str)
        self.assertEqual(t.body[1].lineno, 3)
        self.assertEqual(t.body[1].value.lineno, 3)
        self.assertEqual(t.body[1].value.values[0].lineno, 3)
        self.assertEqual(t.body[1].value.values[1].lineno, 4)
        self.assertEqual(t.body[1].value.values[2].lineno, 6)
        self.assertEqual(t.body[1].col_offset, 0)
        self.assertEqual(t.body[1].value.col_offset, 0)
        self.assertEqual(t.body[1].value.values[0].col_offset, 4)
        self.assertEqual(t.body[1].value.values[1].col_offset, 2)
        self.assertEqual(t.body[1].value.values[2].col_offset, 11)
        # NOTE: the following lineno information and col_offset is correct for
        # expressions within FormattedValues.
        binop = t.body[1].value.values[1].value
        self.assertEqual(type(binop), ast.BinOp)
        self.assertEqual(type(binop.left), ast.Name)
        self.assertEqual(type(binop.op), ast.Mult)
        self.assertEqual(type(binop.right), ast.Call)
        self.assertEqual(binop.lineno, 4)
        self.assertEqual(binop.left.lineno, 4)
        self.assertEqual(binop.right.lineno, 6)
        self.assertEqual(binop.col_offset, 3)
        self.assertEqual(binop.left.col_offset, 3)
        self.assertEqual(binop.right.col_offset, 7)

        expr = """
a = f'''
          {blech}
    '''
"""
        t = ast.parse(expr)
        self.assertEqual(type(t), ast.Module)
        self.assertEqual(len(t.body), 1)
        # Check f'...'
        self.assertEqual(type(t.body[0]), ast.Assign)
        self.assertEqual(type(t.body[0].value), ast.JoinedStr)
        self.assertEqual(len(t.body[0].value.values), 3)
        self.assertEqual(type(t.body[0].value.values[1]), ast.FormattedValue)
        self.assertEqual(t.body[0].lineno, 2)
        self.assertEqual(t.body[0].value.lineno, 2)
        self.assertEqual(t.body[0].value.values[0].lineno, 2)
        self.assertEqual(t.body[0].value.values[1].lineno, 3)
        self.assertEqual(t.body[0].value.values[2].lineno, 3)
        self.assertEqual(t.body[0].col_offset, 0)
        self.assertEqual(t.body[0].value.col_offset, 4)
        self.assertEqual(t.body[0].value.values[0].col_offset, 8)
        self.assertEqual(t.body[0].value.values[1].col_offset, 10)
        self.assertEqual(t.body[0].value.values[2].col_offset, 17)
        # Check {blech}
        self.assertEqual(t.body[0].value.values[1].value.lineno, 3)
        self.assertEqual(t.body[0].value.values[1].value.end_lineno, 3)
        self.assertEqual(t.body[0].value.values[1].value.col_offset, 11)
        self.assertEqual(t.body[0].value.values[1].value.end_col_offset, 16)

    def __test_ast_line_numbers_with_parentheses(self):
        expr = """
x = (
    f" {test(t)}"
)"""
        t = ast.parse(expr)
        self.assertEqual(type(t), ast.Module)
        self.assertEqual(len(t.body), 1)
        # check the joinedstr location
        joinedstr = t.body[0].value
        self.assertEqual(type(joinedstr), ast.JoinedStr)
        self.assertEqual(joinedstr.lineno, 3)
        self.assertEqual(joinedstr.end_lineno, 3)
        self.assertEqual(joinedstr.col_offset, 4)
        self.assertEqual(joinedstr.end_col_offset, 17)
        # check the formatted value location
        fv = t.body[0].value.values[1]
        self.assertEqual(type(fv), ast.FormattedValue)
        self.assertEqual(fv.lineno, 3)
        self.assertEqual(fv.end_lineno, 3)
        self.assertEqual(fv.col_offset, 7)
        self.assertEqual(fv.end_col_offset, 16)
        # check the test(t) location
        call = t.body[0].value.values[1].value
        self.assertEqual(type(call), ast.Call)
        self.assertEqual(call.lineno, 3)
        self.assertEqual(call.end_lineno, 3)
        self.assertEqual(call.col_offset, 8)
        self.assertEqual(call.end_col_offset, 15)

        expr = """
x = (
    u'wat',
    u"wat",
    b'wat',
    b"wat",
    f'wat',
    f"wat",
)

y = (
    u'''wat''',
    u\"\"\"wat\"\"\",
    b'''wat''',
    b\"\"\"wat\"\"\",
    f'''wat''',
    f\"\"\"wat\"\"\",
)
        """
        t = ast.parse(expr)
        self.assertEqual(type(t), ast.Module)
        self.assertEqual(len(t.body), 2)
        x, y = t.body

        # Check the single quoted string offsets first.
        offsets = [
            (elt.col_offset, elt.end_col_offset)
            for elt in x.value.elts
        ]
        self.assertTrue(all(
            offset == (4, 10)
            for offset in offsets
        ))

        # Check the triple quoted string offsets.
        offsets = [
            (elt.col_offset, elt.end_col_offset)
            for elt in y.value.elts
        ]
        self.assertTrue(all(
            offset == (4, 14)
            for offset in offsets
        ))

        expr = """
    x = (
            'PERL_MM_OPT', (
                f'wat'
                f'some_string={f(x)} '
                f'wat'
            ),
    )
    """
        t = ast.parse(expr)
        self.assertEqual(type(t), ast.Module)
        self.assertEqual(len(t.body), 1)
        # check the fstring
        fstring = t.body[0].value.elts[1]
        self.assertEqual(type(fstring), ast.JoinedStr)
        self.assertEqual(len(fstring.values), 3)
        wat1, middle, wat2 = fstring.values
        # check the first wat
        self.assertEqual(type(wat1), ast.Constant)
        self.assertEqual(wat1.lineno, 4)
        self.assertEqual(wat1.end_lineno, 5)
        self.assertEqual(wat1.col_offset, 14)
        self.assertEqual(wat1.end_col_offset, 26)
        # check the call
        call = middle.value
        self.assertEqual(type(call), ast.Call)
        self.assertEqual(call.lineno, 5)
        self.assertEqual(call.end_lineno, 5)
        self.assertEqual(call.col_offset, 27)
        self.assertEqual(call.end_col_offset, 31)
        # check the second wat
        self.assertEqual(type(wat2), ast.Constant)
        self.assertEqual(wat2.lineno, 5)
        self.assertEqual(wat2.end_lineno, 6)
        self.assertEqual(wat2.col_offset, 32)
        # wat ends at the offset 17, but the whole f-string
        # ends at the offset 18 (since the quote is part of the
        # f-string but not the wat string)
        self.assertEqual(wat2.end_col_offset, 17)
        self.assertEqual(fstring.end_col_offset, 18)

    def __test_ast_fstring_empty_format_spec(self):
        expr = "f'{expr:}'"

        mod = ast.parse(expr)
        self.assertEqual(type(mod), ast.Module)
        self.assertEqual(len(mod.body), 1)

        fstring = mod.body[0].value
        self.assertEqual(type(fstring), ast.JoinedStr)
        self.assertEqual(len(fstring.values), 1)

        fv = fstring.values[0]
        self.assertEqual(type(fv), ast.FormattedValue)

        format_spec = fv.format_spec
        self.assertEqual(type(format_spec), ast.JoinedStr)
        self.assertEqual(len(format_spec.values), 0)

    def test_docstring(self):
        def f():
            f'''Not a docstring'''
        self.assertIsNone(f.__doc__)
        def g():
            '''Not a docstring''' \
            f''
        self.assertIsNone(g.__doc__)

    def __test_literal_eval(self):
        with self.assertRaisesRegex(ValueError, 'malformed node or string'):
            ast.literal_eval("f'x'")

    def __test_ast_compile_time_concat(self):
        x = ['']

        expr = """x[0] = 'foo' f'{3}'"""
        t = ast.parse(expr)
        c = compile(t, '', 'exec')
        exec(c)
        self.assertEqual(x[0], 'foo3')

    def test_compile_time_concat_errors(self):
        self.assertAllRaise(SyntaxError,
                            'cannot mix bytes and nonbytes literals',
                            [r"""f'' b''""",
                             r"""b'' f''""",
                             ])

    def test_literal(self):
        self.assertEqual(f'', '')
        self.assertEqual(f'a', 'a')
        self.assertEqual(f' ', ' ')

    def test_unterminated_string(self):
        self.assertAllRaise(SyntaxError, 'unterminated string',
                            [r"""f'{"x'""",
                             r"""f'{"x}'""",
                             r"""f'{("x'""",
                             r"""f'{("x}'""",
                             ])

    #@unittest.skipIf(support.is_wasi, "exhausts limited stack on WASI")
    def test_mismatched_parens(self):
        self.assertAllRaise(SyntaxError, r"closing parenthesis '\}' "
                            r"does not match opening parenthesis '\('",
                            ["f'{((}'",
                             ])
        self.assertAllRaise(SyntaxError, r"closing parenthesis '\)' "
                            r"does not match opening parenthesis '\['",
                            ["f'{a[4)}'",
                            ])
        self.assertAllRaise(SyntaxError, r"closing parenthesis '\]' "
                            r"does not match opening parenthesis '\('",
                            ["f'{a(4]}'",
                            ])
        self.assertAllRaise(SyntaxError, r"closing parenthesis '\}' "
                            r"does not match opening parenthesis '\['",
                            ["f'{a[4}'",
                            ])
        self.assertAllRaise(SyntaxError, r"closing parenthesis '\}' "
                            r"does not match opening parenthesis '\('",
                            ["f'{a(4}'",
                            ])
        # disabled: Used to test CPython and the nesting is too deep for Cython's parser.
        #self.assertRaises(SyntaxError, eval, "f'{" + "("*500 + "}'")

    #@unittest.skipIf(support.is_wasi, "exhausts limited stack on WASI")
    def __test_fstring_nested_too_deeply(self):
        self.assertAllRaise(SyntaxError,
                            "f-string: expressions nested too deeply",
                            ['f"{1+2:{1+2:{1+1:{1}}}}"'])

        def create_nested_fstring(n):
            if n == 0:
                return "1+1"
            prev = create_nested_fstring(n - 1)
            return f'f"{{{prev}}}"'

        self.assertAllRaise(SyntaxError,
                            "too many nested f-strings",
                            [create_nested_fstring(160)])

    # Cython hits the recursion limit here. If you raise the recursion limit
    # then the test is fine though.
    def __test_syntax_error_in_nested_fstring(self):
        # See gh-104016 for more information on this crash
        self.assertAllRaise(SyntaxError,
                            "invalid syntax",
                            ['f"{1 1:' + ('{f"1:' * 199)])

    def test_double_braces(self):
        self.assertEqual(f'{{', '{')
        self.assertEqual(f'a{{', 'a{')
        self.assertEqual(f'{{b', '{b')
        self.assertEqual(f'a{{b', 'a{b')
        self.assertEqual(f'}}', '}')
        self.assertEqual(f'a}}', 'a}')
        self.assertEqual(f'}}b', '}b')
        self.assertEqual(f'a}}b', 'a}b')
        self.assertEqual(f'{{}}', '{}')
        self.assertEqual(f'a{{}}', 'a{}')
        self.assertEqual(f'{{b}}', '{b}')
        self.assertEqual(f'{{}}c', '{}c')
        self.assertEqual(f'a{{b}}', 'a{b}')
        self.assertEqual(f'a{{}}c', 'a{}c')
        self.assertEqual(f'{{b}}c', '{b}c')
        self.assertEqual(f'a{{b}}c', 'a{b}c')

        self.assertEqual(f'{{{10}', '{10')
        self.assertEqual(f'}}{10}', '}10')
        self.assertEqual(f'}}{{{10}', '}{10')
        self.assertEqual(f'}}a{{{10}', '}a{10')

        self.assertEqual(f'{10}{{', '10{')
        self.assertEqual(f'{10}}}', '10}')
        self.assertEqual(f'{10}}}{{', '10}{')
        self.assertEqual(f'{10}}}a{{' '}', '10}a{}')

        # Inside of strings, don't interpret doubled brackets.
        self.assertEqual(f'{"{{}}"}', '{{}}')

        self.assertAllRaise(TypeError, 'unhashable type',
                            ["f'{ {{}} }'", # dict in a set
                             ])

    def test_compile_time_concat(self):
        x = 'def'
        self.assertEqual('abc' f'## {x}ghi', 'abc## defghi')
        self.assertEqual('abc' f'{x}' 'ghi', 'abcdefghi')
        self.assertEqual('abc' f'{x}' 'gh' f'i{x:4}', 'abcdefghidef ')
        self.assertEqual('{x}' f'{x}', '{x}def')
        self.assertEqual('{x' f'{x}', '{xdef')
        self.assertEqual('{x}' f'{x}', '{x}def')
        self.assertEqual('{{x}}' f'{x}', '{{x}}def')
        self.assertEqual('{{x' f'{x}', '{{xdef')
        self.assertEqual('x}}' f'{x}', 'x}}def')
        self.assertEqual(f'{x}' 'x}}', 'defx}}')
        self.assertEqual(f'{x}' '', 'def')
        self.assertEqual('' f'{x}' '', 'def')
        self.assertEqual('' f'{x}', 'def')
        self.assertEqual(f'{x}' '2', 'def2')
        self.assertEqual('1' f'{x}' '2', '1def2')
        self.assertEqual('1' f'{x}', '1def')
        self.assertEqual(f'{x}' f'-{x}', 'def-def')
        self.assertEqual('' f'', '')
        self.assertEqual('' f'' '', '')
        self.assertEqual('' f'' '' f'', '')
        self.assertEqual(f'', '')
        self.assertEqual(f'' '', '')
        self.assertEqual(f'' '' f'', '')
        self.assertEqual(f'' '' f'' '', '')

        # This is not really [f'{'] + [f'}'] since we treat the inside
        # of braces as a purely new context, so it is actually f'{ and
        # then eval('  f') (a valid expression) and then }' which would
        # constitute a valid f-string.
        self.assertEqual(f'{' f'}', ' f')

        self.assertAllRaise(SyntaxError, "expecting '}'",
                            ['''f'{3' f"}"''',  # can't concat to get a valid f-string
                             ])

    def test_comments(self):
        # These aren't comments, since they're in strings.
        d = {'#': 'hash'}
        self.assertEqual(f'{"#"}', '#')
        self.assertEqual(f'{d["#"]}', 'hash')

        self.assertAllRaise(SyntaxError, "'{' was never closed",
                            ["f'{1#}'",   # error because everything after '#' is a comment
                             "f'{#}'",
                             "f'one: {1#}'",
                             "f'{1# one} {2 this is a comment still#}'",
                             ])
        self.assertAllRaise(SyntaxError, r"f-string: unmatched '\)'",
                            ["f'{)#}'",   # When wrapped in parens, this becomes
                                          #  '()#)'.  Make sure that doesn't compile.
                             ])
        self.assertEqual(f'''A complex trick: {
2  # two
}''', 'A complex trick: 2')
        self.assertEqual(f'''
{
40 # forty
+  # plus
2  # two
}''', '\n42')
        self.assertEqual(f'''
{
40 # forty
+  # plus
2  # two
}''', '\n42')

        self.assertEqual(f'''
# this is not a comment
{ # the following operation it's
3 # this is a number
* 2}''', '\n# this is not a comment\n6')
        self.assertEqual(f'''
{# f'a {comment}'
86 # constant
# nothing more
}''', '\n86')

        self.assertAllRaise(SyntaxError, r"f-string: valid expression required before '}'",
                            ["""f'''
{
# only a comment
}'''
""", # this is equivalent to f'{}'
                             ])

    def test_many_expressions(self):
        # Create a string with many expressions in it. Note that
        #  because we have a space in here as a literal, we're actually
        #  going to use twice as many ast nodes: one for each literal
        #  plus one for each expression.
        def build_fstr(n, extra=''):
            return "f'" + ('{x} ' * n) + extra + "'"

        x = 'X'
        width = 1

        # Test around 256.
        # for i in range(250, 260):
        #     self.assertEqual(eval(build_fstr(i)), (x+' ')*i)
        self.assertEqual(
            cy_eval('[' + ', '.join(build_fstr(i) for i in range(250, 260)) + ']', x=x, width=width),
            [(x+' ')*i for i in range(250, 260)],
        )

        # Test concatenating 2 largs fstrings.
        # self.assertEqual(eval(build_fstr(255)*256), (x+' ')*(255*256))
        self.assertEqual(cy_eval(build_fstr(255)*3, x=x, width=width), (x+' ')*(255*3))  # CPython uses 255*256

        s = build_fstr(253, '{x:{width}} ')
        # self.assertEqual(eval(s), (x+' ')*254)
        self.assertEqual(cy_eval(s, x=x, width=width), (x+' ')*254)

        # Test lots of expressions and constants, concatenated.
        s = "f'{1}' 'x' 'y'" * 1024
        # self.assertEqual(eval(s), '1xy' * 1024)
        self.assertEqual(cy_eval(s, x=x, width=width), '1xy' * 1024)

    def test_format_specifier_expressions(self):
        width = 10
        precision = 4
        value = decimal.Decimal('12.34567')
        self.assertEqual(f'result: {value:{width}.{precision}}', 'result:      12.35')
        self.assertEqual(f'result: {value:{width!r}.{precision}}', 'result:      12.35')
        self.assertEqual(f'result: {value:{width:0}.{precision:1}}', 'result:      12.35')
        self.assertEqual(f'result: {value:{1}{0:0}.{precision:1}}', 'result:      12.35')
        self.assertEqual(f'result: {value:{ 1}{ 0:0}.{ precision:1}}', 'result:      12.35')
        self.assertEqual(f'{10:#{1}0x}', '       0xa')
        self.assertEqual(f'{10:{"#"}1{0}{"x"}}', '       0xa')
        self.assertEqual(f'{-10:-{"#"}1{0}x}', '      -0xa')
        self.assertEqual(f'{-10:{"-"}#{1}0{"x"}}', '      -0xa')
        self.assertEqual(f'{10:#{3 != {4:5} and width}x}', '       0xa')
        self.assertEqual(f'result: {value:{width:{0}}.{precision:1}}', 'result:      12.35')

        self.assertAllRaise(SyntaxError, "f-string: expecting ':' or '}'",
                            ["""f'{"s"!r{":10"}}'""",
                             # This looks like a nested format spec.
                             ])

        self.assertAllRaise(SyntaxError,
                            "f-string: expecting a valid expression after '{'",
                            [# Invalid syntax inside a nested spec.
                             "f'{4:{/5}}'",
                             ])

        self.assertAllRaise(SyntaxError, 'f-string: invalid conversion character',
                            [# No expansion inside conversion or for
                             #  the : or ! itself.
                             """f'{"s"!{"r"}}'""",
                             ])

    def test_custom_format_specifier(self):
        class CustomFormat:
            def __format__(self, format_spec):
                return format_spec

        self.assertEqual(f'{CustomFormat():\n}', '\n')
        self.assertEqual(f'{CustomFormat():\u2603}', '☃')
        #with self.assertWarns(SyntaxWarning):
        #    exec(r'f"{F():¯\_(ツ)_/¯}"', {'F': CustomFormat})

    def test_side_effect_order(self):
        class X:
            def __init__(self):
                self.i = 0
            def __format__(self, spec):
                self.i += 1
                return str(self.i)

        x = X()
        self.assertEqual(f'{x} {x}', '1 2')

    def test_missing_expression(self):
        self.assertAllRaise(SyntaxError,
                            "f-string: valid expression required before '}'",
                            ["f'{}'",
                             "f'{ }'"
                             "f' {} '",
                             "f'{10:{ }}'",
                             "f' { } '",

                             # The Python parser ignores also the following
                             # whitespace characters in additional to a space.
                             "f'''{\t\f\r\n}'''",
                             ])

        self.assertAllRaise(SyntaxError,
                            "f-string: valid expression required before '!'",
                            ["f'{!r}'",
                             "f'{ !r}'",
                             "f'{!}'",
                             "f'''{\t\f\r\n!a}'''",

                             # Catch empty expression before the
                             #  missing closing brace.
                             "f'{!'",
                             "f'{!s:'",

                             # Catch empty expression before the
                             #  invalid conversion.
                             "f'{!x}'",
                             "f'{ !xr}'",
                             "f'{!x:}'",
                             "f'{!x:a}'",
                             "f'{ !xr:}'",
                             "f'{ !xr:a}'",
                             ])

        self.assertAllRaise(SyntaxError,
                            "f-string: valid expression required before ':'",
                            ["f'{:}'",
                             "f'{ :!}'",
                             "f'{:2}'",
                             "f'''{\t\f\r\n:a}'''",
                             "f'{:'",
                             ])

        self.assertAllRaise(SyntaxError,
                            "f-string: valid expression required before '='",
                            ["f'{=}'",
                             "f'{ =}'",
                             "f'{ =:}'",
                             "f'{   =!}'",
                             "f'''{\t\f\r\n=}'''",
                             "f'{='",
                             ])

        # Different error message is raised for other whitespace characters.
        self.assertAllRaise(SyntaxError, r"invalid non-printable character U\+00A0",
                            ["f'''{\xa0}'''",
                             #"\xa0",
                             ])

    def test_parens_in_expressions(self):
        self.assertEqual(f'{3,}', '(3,)')

        self.assertAllRaise(SyntaxError,
                            "f-string: expecting a valid expression after '{'",
                            ["f'{,}'",
                             ])

        self.assertAllRaise(SyntaxError, r"f-string: unmatched '\)'",
                            ["f'{3)+(4}'",
                             ])

    def test_newlines_before_syntax_error(self):
        self.assertAllRaise(SyntaxError,
                            "f-string: expecting a valid expression after '{'",
                            ["f'{.}'", "\nf'{.}'", "\n\nf'{.}'"])

    def test_backslashes_in_string_part(self):
        self.assertEqual(f'\t', '\t')
        self.assertEqual(r'\t', '\\t')
        self.assertEqual(rf'\t', '\\t')
        self.assertEqual(f'{2}\t', '2\t')
        self.assertEqual(f'{2}\t{3}', '2\t3')
        self.assertEqual(f'\t{3}', '\t3')

        self.assertEqual(f'\u0394', '\u0394')
        self.assertEqual(r'\u0394', '\\u0394')
        self.assertEqual(rf'\u0394', '\\u0394')
        self.assertEqual(f'{2}\u0394', '2\u0394')
        self.assertEqual(f'{2}\u0394{3}', '2\u03943')
        self.assertEqual(f'\u0394{3}', '\u03943')

        self.assertEqual(f'\U00000394', '\u0394')
        self.assertEqual(r'\U00000394', '\\U00000394')
        self.assertEqual(rf'\U00000394', '\\U00000394')
        self.assertEqual(f'{2}\U00000394', '2\u0394')
        self.assertEqual(f'{2}\U00000394{3}', '2\u03943')
        self.assertEqual(f'\U00000394{3}', '\u03943')

        self.assertEqual(f'\N{GREEK CAPITAL LETTER DELTA}', '\u0394')
        self.assertEqual(f'{2}\N{GREEK CAPITAL LETTER DELTA}', '2\u0394')
        self.assertEqual(f'{2}\N{GREEK CAPITAL LETTER DELTA}{3}', '2\u03943')
        self.assertEqual(f'\N{GREEK CAPITAL LETTER DELTA}{3}', '\u03943')
        self.assertEqual(f'2\N{GREEK CAPITAL LETTER DELTA}', '2\u0394')
        self.assertEqual(f'2\N{GREEK CAPITAL LETTER DELTA}3', '2\u03943')
        self.assertEqual(f'\N{GREEK CAPITAL LETTER DELTA}3', '\u03943')

        self.assertEqual(f'\x20', ' ')
        self.assertEqual(r'\x20', '\\x20')
        self.assertEqual(rf'\x20', '\\x20')
        self.assertEqual(f'{2}\x20', '2 ')
        self.assertEqual(f'{2}\x20{3}', '2 3')
        self.assertEqual(f'\x20{3}', ' 3')

        self.assertEqual(f'2\x20', '2 ')
        self.assertEqual(f'2\x203', '2 3')
        self.assertEqual(f'\x203', ' 3')

        #with self.assertWarns(SyntaxWarning):  # invalid escape sequence
        #    value = cy_eval(r"f'\{6*7}'")
        #self.assertEqual(value, '\\42')
        #with self.assertWarns(SyntaxWarning):  # invalid escape sequence
        #    value = eval(r"f'\g'")
        #self.assertEqual(value, '\\g')
        self.assertEqual(f'\\{6*7}', '\\42')
        self.assertEqual(fr'\{6*7}', '\\42')

        AMPERSAND = 'spam'
        # Get the right unicode character (&), or pick up local variable
        # depending on the number of backslashes.
        self.assertEqual(f'\N{AMPERSAND}', '&')
        self.assertEqual(f'\\N{AMPERSAND}', '\\Nspam')
        self.assertEqual(fr'\N{AMPERSAND}', '\\Nspam')
        self.assertEqual(f'\\\N{AMPERSAND}', '\\&')

    def test_misformed_unicode_character_name(self):
        # These test are needed because unicode names are parsed
        # differently inside f-strings.
        self.assertAllRaise(SyntaxError, r"\(unicode error\) 'unicodeescape' codec can't decode bytes in position .*: malformed \\N character escape",
                            [r"f'\N'",
                             r"f'\N '",
                             r"f'\N  '",  # See bpo-46503.
                             r"f'\N{'",
                             r"f'\N{GREEK CAPITAL LETTER DELTA'",

                             # Here are the non-f-string versions,
                             #  which should give the same errors.
                             r"'\N'",
                             r"'\N '",
                             r"'\N  '",
                             r"'\N{'",
                             r"'\N{GREEK CAPITAL LETTER DELTA'",
                             ])

    def test_backslashes_in_expression_part(self):
        self.assertEqual(f"{(
                        1 +
                        2
        )}", "3")

        self.assertEqual("\N{LEFT CURLY BRACKET}", '{')
        self.assertEqual(f'{"\N{LEFT CURLY BRACKET}"}', '{')
        self.assertEqual(rf'{"\N{LEFT CURLY BRACKET}"}', '{')

        self.assertAllRaise(SyntaxError,
                            "f-string: valid expression required before '}'",
                            ["f'{\n}'",
                             ])

    def test_invalid_backslashes_inside_fstring_context(self):
        # All of these variations are invalid python syntax,
        # so they are also invalid in f-strings as well.
        cases = [
            formatting.format(expr=expr)
            for formatting in [
                "{expr}",
                "f'{{{expr}}}'",
                "rf'{{{expr}}}'",
            ]
            for expr in [
                r"\'a\'",
                r"\t3",
                r"\\"[0],
            ]
        ]
        self.assertAllRaise(SyntaxError, 'unexpected character after line continuation',
                            cases)

    def test_no_escapes_for_braces(self):
        """
        Only literal curly braces begin an expression.
        """
        # \x7b is '{'.
        self.assertEqual(f'\x7b1+1}}', '{1+1}')
        self.assertEqual(f'\x7b1+1', '{1+1')
        self.assertEqual(f'\u007b1+1', '{1+1')
        self.assertEqual(f'\N{LEFT CURLY BRACKET}1+1\N{RIGHT CURLY BRACKET}', '{1+1}')

    def test_newlines_in_expressions(self):
        self.assertEqual(f'{0}', '0')
        self.assertEqual(rf'''{3+
4}''', '7')

    def test_lambda(self):
        x = 5
        self.assertEqual(f'{(lambda y:x*y)("8")!r}', "'88888'")
        self.assertEqual(f'{(lambda y:x*y)("8")!r:10}', "'88888'   ")
        self.assertEqual(f'{(lambda y:x*y)("8"):10}', "88888     ")

        # lambda doesn't work without parens, because the colon
        # makes the parser think it's a format_spec
        # emit warning if we can match a format_spec
        self.assertAllRaise(SyntaxError,
                            "f-string: lambda expressions are not allowed "
                            "without parentheses",
                            ["f'{lambda x:x}'",
                             "f'{lambda :x}'",
                             "f'{lambda *arg, :x}'",
                             "f'{1, lambda:x}'",
                             "f'{lambda x:}'",
                             "f'{lambda :}'",
                             ])

        # but don't emit the paren warning in general cases
        with self.assertRaisesRegex(SyntaxError, "f-string: expecting a valid expression after '{'"):
            cy_eval("f'{+ lambda:None}'")

    def test_valid_prefixes(self):
        self.assertEqual(F'{1}', "1")
        self.assertEqual(FR'{2}', "2")
        self.assertEqual(fR'{3}', "3")

    def test_roundtrip_raw_quotes(self):
        self.assertEqual(fr"\'", "\\'")
        self.assertEqual(fr'\"', '\\"')
        self.assertEqual(fr'\"\'', '\\"\\\'')
        self.assertEqual(fr'\'\"', '\\\'\\"')
        self.assertEqual(fr'\"\'\"', '\\"\\\'\\"')
        self.assertEqual(fr'\'\"\'', '\\\'\\"\\\'')
        self.assertEqual(fr'\"\'\"\'', '\\"\\\'\\"\\\'')

    def test_fstring_backslash_before_double_bracket(self):
        deprecated_cases = [
            (r"f'\{{\}}'", '\\{\\}'),
            (r"f'\{{'", '\\{'),
            (r"f'\{{{1+1}'", '\\{2'),
            (r"f'\}}{1+1}'", '\\}2'),
            (r"f'{1+1}\}}'", '2\\}')
        ]
        #for case, expected_result in deprecated_cases:
        #    with self.subTest(case=case, expected_result=expected_result):
        #        with self.assertWarns(SyntaxWarning):
        #            result = eval(case)
        #        result = self.
        #        self.assertEqual(result, expected_result)
        self.assertEqual(fr'\{{\}}', '\\{\\}')
        self.assertEqual(fr'\{{', '\\{')
        self.assertEqual(fr'\{{{1 + 1}', '\\{2')
        self.assertEqual(fr'\}}{1 + 1}', '\\}2')
        self.assertEqual(fr'{1 + 1}\}}', '2\\}')

    def __test_fstring_backslash_before_double_bracket_warns_once(self):
        with self.assertWarns(SyntaxWarning) as w:
            eval(r"f'\{{'")
        self.assertEqual(len(w.warnings), 1)
        self.assertEqual(w.warnings[0].category, SyntaxWarning)

    def test_fstring_backslash_prefix_raw(self):
        self.assertEqual(f'\\', '\\')
        self.assertEqual(f'\\\\', '\\\\')
        self.assertEqual(fr'\\', r'\\')
        self.assertEqual(fr'\\\\', r'\\\\')
        self.assertEqual(rf'\\', r'\\')
        self.assertEqual(rf'\\\\', r'\\\\')
        self.assertEqual(Rf'\\', R'\\')
        self.assertEqual(Rf'\\\\', R'\\\\')
        self.assertEqual(fR'\\', R'\\')
        self.assertEqual(fR'\\\\', R'\\\\')
        self.assertEqual(FR'\\', R'\\')
        self.assertEqual(FR'\\\\', R'\\\\')

    def test_fstring_format_spec_greedy_matching(self):
        self.assertEqual(f"{1:}}}", "1}")
        self.assertEqual(f"{1:>3{5}}}}", "                                  1}")

    def test_yield(self):
        # Not terribly useful, but make sure the yield turns
        #  a function into a generator
        def fn(y):
            f'y:{yield y*2}'
            f'{yield}'

        g = fn(4)
        self.assertEqual(next(g), 8)
        self.assertEqual(next(g), None)

    def test_yield_send(self):
        def fn(x):
            yield f'x:{yield (lambda i: x * i)}'

        g = fn(10)
        the_lambda = next(g)
        self.assertEqual(the_lambda(4), 40)
        self.assertEqual(g.send('string'), 'x:string')

    def test_expressions_with_triple_quoted_strings(self):
        self.assertEqual(f"{'''x'''}", 'x')
        self.assertEqual(f"{'''eric's'''}", "eric's")

        # Test concatenation within an expression
        self.assertEqual(f'{"x" """eric"s""" "y"}', 'xeric"sy')
        self.assertEqual(f'{"x" """eric"s"""}', 'xeric"s')
        self.assertEqual(f'{"""eric"s""" "y"}', 'eric"sy')
        self.assertEqual(f'{"""x""" """eric"s""" "y"}', 'xeric"sy')
        self.assertEqual(f'{"""x""" """eric"s""" """y"""}', 'xeric"sy')
        self.assertEqual(f'{r"""x""" """eric"s""" """y"""}', 'xeric"sy')

    def test_multiple_vars(self):
        x = 98
        y = 'abc'
        self.assertEqual(f'{x}{y}', '98abc')

        self.assertEqual(f'X{x}{y}', 'X98abc')
        self.assertEqual(f'{x}X{y}', '98Xabc')
        self.assertEqual(f'{x}{y}X', '98abcX')

        self.assertEqual(f'X{x}Y{y}', 'X98Yabc')
        self.assertEqual(f'X{x}{y}Y', 'X98abcY')
        self.assertEqual(f'{x}X{y}Y', '98XabcY')

        self.assertEqual(f'X{x}Y{y}Z', 'X98YabcZ')

    def test_closure(self):
        def outer(x):
            def inner():
                return f'x:{x}'
            return inner

        self.assertEqual(outer('987')(), 'x:987')
        self.assertEqual(outer(7)(), 'x:7')

    def test_arguments(self):
        y = 2
        def f(x, width):
            return f'x={x*y:{width}}'

        self.assertEqual(f('foo', 10), 'x=foofoo    ')
        x = 'bar'
        self.assertEqual(f(10, 10), 'x=        20')

    def test_locals(self):
        value = 123
        self.assertEqual(f'v:{value}', 'v:123')

    def test_missing_variable(self):
        with self.assertRaises(NameError):
            f'v:{value}'

    def test_missing_format_spec(self):
        class O:
            def __format__(self, spec):
                if not spec:
                    return '*'
                return spec

        self.assertEqual(f'{O():x}', 'x')
        self.assertEqual(f'{O()}', '*')
        self.assertEqual(f'{O():}', '*')

        self.assertEqual(f'{3:}', '3')
        self.assertEqual(f'{3!s:}', '3')

    def test_global(self):
        self.assertEqual(f'g:{a_global}', 'g:global variable')
        self.assertEqual(f'g:{a_global!r}', "g:'global variable'")

        a_local = 'local variable'
        self.assertEqual(f'g:{a_global} l:{a_local}',
                         'g:global variable l:local variable')
        self.assertEqual(f'g:{a_global!r}',
                         "g:'global variable'")
        self.assertEqual(f'g:{a_global} l:{a_local!r}',
                         "g:global variable l:'local variable'")

        self.assertIn("module 'unittest' from", f'{unittest}')

    def test_shadowed_global(self):
        a_global = 'really a local'
        self.assertEqual(f'g:{a_global}', 'g:really a local')
        self.assertEqual(f'g:{a_global!r}', "g:'really a local'")

        a_local = 'local variable'
        self.assertEqual(f'g:{a_global} l:{a_local}',
                         'g:really a local l:local variable')
        self.assertEqual(f'g:{a_global!r}',
                         "g:'really a local'")
        self.assertEqual(f'g:{a_global} l:{a_local!r}',
                         "g:really a local l:'local variable'")

    def test_call(self):
        def foo(x):
            return 'x=' + str(x)

        self.assertEqual(f'{foo(10)}', 'x=10')

    def test_nested_fstrings(self):
        y = 5
        self.assertEqual(f'{f"{0}"*3}', '000')
        self.assertEqual(f'{f"{y}"*3}', '555')

    def test_invalid_string_prefixes(self):
        single_quote_cases = ["fu''",
                             "uf''",
                             "Fu''",
                             "fU''",
                             "Uf''",
                             "uF''",
                             "ufr''",
                             "urf''",
                             "fur''",
                             "fru''",
                             "rfu''",
                             "ruf''",
                             "FUR''",
                             "Fur''",
                             "fb''",
                             "fB''",
                             "Fb''",
                             "FB''",
                             "bf''",
                             "bF''",
                             "Bf''",
                             "BF''",]
        double_quote_cases = [case.replace("'", '"') for case in single_quote_cases]
        self.assertAllRaise(SyntaxError, 'invalid syntax',
                            single_quote_cases + double_quote_cases)

    def test_leading_trailing_spaces(self):
        self.assertEqual(f'{ 3}', '3')
        self.assertEqual(f'{  3}', '3')
        self.assertEqual(f'{3 }', '3')
        self.assertEqual(f'{3  }', '3')

        self.assertEqual(f'expr={ {x: y for x, y in [(1, 2), ]}}',
                         'expr={1: 2}')
        self.assertEqual(f'expr={ {x: y for x, y in [(1, 2), ]} }',
                         'expr={1: 2}')

    def test_not_equal(self):
        # There's a special test for this because there's a special
        #  case in the f-string parser to look for != as not ending an
        #  expression. Normally it would, while looking for !s or !r.

        self.assertEqual(f'{3!=4}', 'True')
        self.assertEqual(f'{3!=4:}', 'True')
        self.assertEqual(f'{3!=4!s}', 'True')
        self.assertEqual(f'{3!=4!s:.3}', 'Tru')

    def test_equal_equal(self):
        # Because an expression ending in = has special meaning,
        # there's a special test for ==. Make sure it works.

        self.assertEqual(f'{0==1}', 'False')

    def test_conversions(self):
        self.assertEqual(f'{3.14:10.10}', '      3.14')
        self.assertEqual(f'{3.14!s:10.10}', '3.14      ')
        self.assertEqual(f'{3.14!r:10.10}', '3.14      ')
        self.assertEqual(f'{3.14!a:10.10}', '3.14      ')

        self.assertEqual(f'{"a"}', 'a')
        self.assertEqual(f'{"a"!r}', "'a'")
        self.assertEqual(f'{"a"!a}', "'a'")

        # Conversions can have trailing whitespace after them since it
        # does not provide any significance
        self.assertEqual(f"{3!s  }", "3")
        self.assertEqual(f'{3.14!s  :10.10}', '3.14      ')

        # Not a conversion.
        self.assertEqual(f'{"a!r"}', "a!r")

        # Not a conversion, but show that ! is allowed in a format spec.
        self.assertEqual(f'{3.14:!<10.10}', '3.14!!!!!!')

        self.assertAllRaise(SyntaxError, "f-string: expecting '}'",
                            ["f'{3!'",
                             "f'{3!s'",
                             "f'{3!g'",
                             ])

        self.assertAllRaise(SyntaxError, 'f-string: missing conversion character',
                            ["f'{3!}'",
                             "f'{3!:'",
                             "f'{3!:}'",
                             ])

        for conv_identifier in 'g', 'A', 'G', 'ä', 'ɐ':
            self.assertAllRaise(SyntaxError,
                                "f-string: invalid conversion character %r: "
                                "expected 's', 'r', or 'a'" % conv_identifier,
                                ["f'{3!" + conv_identifier + "}'"])

        for conv_non_identifier in '3', '!':
            self.assertAllRaise(SyntaxError,
                                "f-string: invalid conversion character",
                                ["f'{3!" + conv_non_identifier + "}'"])

        for conv in ' s', ' s ':
            self.assertAllRaise(SyntaxError,
                                "f-string: conversion type must come right after the"
                                " exclamanation mark",
                                ["f'{3!" + conv + "}'"])

        self.assertAllRaise(SyntaxError,
                            "f-string: invalid conversion character 'ss': "
                            "expected 's', 'r', or 'a'",
                            ["f'{3!ss}'",
                             "f'{3!ss:}'",
                             "f'{3!ss:s}'",
                             ])

    # Cython does get this right, but the error is raised later and not
    # at the parsing stage so the test doesn't catch it.
    # The test in in e_fstring2 instead
    def __test_assignment(self):
        self.assertAllRaise(SyntaxError, r'invalid syntax',
                            ["f'' = 3",
                             "f'{0}' = x",
                             "f'{x}' = x",
                             ])

    def test_del(self):
        self.assertAllRaise(CompileError, 'invalid syntax',   # CPython raises SyntaxError
                            ["del f''",
                             "del '' f''",
                             ])

    def test_mismatched_braces(self):
        self.assertAllRaise(SyntaxError, "f-string: single '}' is not allowed",
                            ["f'{{}'",
                             "f'{{}}}'",
                             "f'}'",
                             "f'x}'",
                             "f'x}x'",
                             r"f'\u007b}'",

                             # Can't have { or } in a format spec.
                             "f'{3:}>10}'",
                             "f'{3:}}>10}'",
                             ])

        self.assertAllRaise(SyntaxError, "f-string: expecting '}'",
                            ["f'{3'",
                             "f'{3!'",
                             "f'{3:'",
                             "f'{3!s'",
                             "f'{3!s:'",
                             "f'{3!s:3'",
                             "f'x{'",
                             "f'x{x'",
                             "f'{x'",
                             "f'{3:s'",
                             "f'{{{'",
                             "f'{{}}{'",
                             "f'{'",
                             "f'{i='",  # See gh-93418.
                             ])

        self.assertAllRaise(SyntaxError,
                            "f-string: expecting a valid expression after '{'",
                            ["f'{3:{{>10}'",
                             ])

        # But these are just normal strings.
        self.assertEqual(f'{"{"}', '{')
        self.assertEqual(f'{"}"}', '}')
        self.assertEqual(f'{3:{"}"}>10}', '}}}}}}}}}3')
        self.assertEqual(f'{2:{"{"}>10}', '{{{{{{{{{2')

    def test_if_conditional(self):
        # There's special logic in compile.c to test if the
        #  conditional for an if (and while) are constants. Exercise
        #  that code.

        def test_fstring(x, expected):
            flag = 0
            if f'{x}':
                flag = 1
            else:
                flag = 2
            self.assertEqual(flag, expected)

        def test_concat_empty(x, expected):
            flag = 0
            if '' f'{x}':
                flag = 1
            else:
                flag = 2
            self.assertEqual(flag, expected)

        def test_concat_non_empty(x, expected):
            flag = 0
            if ' ' f'{x}':
                flag = 1
            else:
                flag = 2
            self.assertEqual(flag, expected)

        test_fstring('', 2)
        test_fstring(' ', 1)

        test_concat_empty('', 2)
        test_concat_empty(' ', 1)

        test_concat_non_empty('', 1)
        test_concat_non_empty(' ', 1)

    def test_empty_format_specifier(self):
        x = 'test'
        self.assertEqual(f'{x}', 'test')
        self.assertEqual(f'{x:}', 'test')
        self.assertEqual(f'{x!s:}', 'test')
        self.assertEqual(f'{x!r:}', "'test'")

    def test_str_format_differences(self):
        d = {'a': 'string',
             0: 'integer',
             }
        a = 0
        self.assertEqual(f'{d[0]}', 'integer')
        self.assertEqual(f'{d["a"]}', 'string')
        self.assertEqual(f'{d[a]}', 'integer')
        self.assertEqual('{d[a]}'.format(d=d), 'string')
        self.assertEqual('{d[0]}'.format(d=d), 'integer')

    def test_errors(self):
        # see issue 26287
        exc = TypeError
        self.assertAllRaise(exc, 'unsupported',
                            [r"f'{(lambda: 0):x}'",
                             r"f'{(0,):x}'",
                             ])
        self.assertAllRaise(ValueError, 'Unknown format code',
                            [r"f'{1000:j}'",
                             r"f'{1000:j}'",
                            ])

    def __test_filename_in_syntaxerror(self):
        # see issue 38964
        with temp_cwd() as cwd:
            file_path = os.path.join(cwd, 't.py')
            with open(file_path, 'w', encoding="utf-8") as f:
                f.write('f"{a b}"')  # This generates a SyntaxError
            _, _, stderr = assert_python_failure(file_path,
                                                 PYTHONIOENCODING='ascii')
        self.assertIn(file_path.encode('ascii', 'backslashreplace'), stderr)

    def test_loop(self):
        for i in range(1000):
            self.assertEqual(f'i:{i}', 'i:' + str(i))

    def test_dict(self):
        d = {'"': 'dquote',
             "'": 'squote',
             'foo': 'bar',
             }
        self.assertEqual(f'''{d["'"]}''', 'squote')
        self.assertEqual(f"""{d['"']}""", 'dquote')

        self.assertEqual(f'{d["foo"]}', 'bar')
        self.assertEqual(f"{d['foo']}", 'bar')

    def __test_backslash_char(self):
        # Check eval of a backslash followed by a control char.
        # See bpo-30682: this used to raise an assert in pydebug mode.
        self.assertEqual(cy_eval('f"\\\n"'), '')
        self.assertEqual(cy_eval('f"\\\r"'), '')

    def test_debug_conversion(self):
        x = 'A string'
        self.assertEqual(f'{x=}', 'x=' + repr(x))
        self.assertEqual(f'{x =}', 'x =' + repr(x))
        self.assertEqual(f'{x=!s}', 'x=' + str(x))
        self.assertEqual(f'{x=!r}', 'x=' + repr(x))
        self.assertEqual(f'{x=!a}', 'x=' + ascii(x))

        x = 2.71828
        self.assertEqual(f'{x=:.2f}', 'x=' + format(x, '.2f'))
        self.assertEqual(f'{x=:}', 'x=' + format(x, ''))
        self.assertEqual(f'{x=!r:^20}', 'x=' + format(repr(x), '^20'))
        self.assertEqual(f'{x=!s:^20}', 'x=' + format(str(x), '^20'))
        self.assertEqual(f'{x=!a:^20}', 'x=' + format(ascii(x), '^20'))

        x = 9
        self.assertEqual(f'{3*x+15=}', '3*x+15=42')

        # There is code in ast.c that deals with non-ascii expression values.  So,
        # use a unicode identifier to trigger that.
        tenπ = 31.4
        self.assertEqual(f'{tenπ=:.2f}', 'tenπ=31.40')

        # Also test with Unicode in non-identifiers.
        self.assertEqual(f'{"Σ"=}', '"Σ"=\'Σ\'')

        # Make sure nested fstrings still work.
        self.assertEqual(f'{f"{3.1415=:.1f}":*^20}', '*****3.1415=3.1*****')

        # Make sure text before and after an expression with = works
        # correctly.
        pi = 'π'
        self.assertEqual(f'alpha α {pi=} ω omega', u"alpha α pi='π' ω omega")

        # Check multi-line expressions.
        self.assertEqual(f'''{
3
=}''', '\n3\n=3')

        # Since = is handled specially, make sure all existing uses of
        # it still work.

        self.assertEqual(f'{0==1}', 'False')
        self.assertEqual(f'{0!=1}', 'True')
        self.assertEqual(f'{0<=1}', 'True')
        self.assertEqual(f'{0>=1}', 'False')
        self.assertEqual(f'{(x:="5")}', '5')
        self.assertEqual(x, '5')
        self.assertEqual(f'{(x:=5)}', '5')
        self.assertEqual(x, 5)
        self.assertEqual(f'{"="}', '=')

        x = 20
        # This isn't an assignment expression, it's 'x', with a format
        # spec of '=10'.  See test_walrus: you need to use parens.
        self.assertEqual(f'{x:=10}', '        20')

        # Test named function parameters, to make sure '=' parsing works
        # there.
        def f(a):
            nonlocal x
            oldx = x
            x = a
            return oldx
        x = 0
        self.assertEqual(f'{f(a="3=")}', '0')
        self.assertEqual(x, '3=')
        self.assertEqual(f'{f(a=4)}', '3=')
        self.assertEqual(x, 4)

        # Make sure __format__ is being called.
        class C:
            def __format__(self, s):
                return f'FORMAT-{s}'
            def __repr__(self):
                return 'REPR'

        self.assertEqual(f'{C()=}', 'C()=REPR')
        self.assertEqual(f'{C()=!r}', 'C()=REPR')
        self.assertEqual(f'{C()=:}', 'C()=FORMAT-')
        self.assertEqual(f'{C()=: }', 'C()=FORMAT- ')
        self.assertEqual(f'{C()=:x}', 'C()=FORMAT-x')
        self.assertEqual(f'{C()=!r:*^20}', 'C()=********REPR********')

        self.assertRaises(SyntaxError, eval, "f'{C=]'")

        # Make sure leading and following text works.
        x = 'foo'
        self.assertEqual(f'X{x=}Y', 'Xx='+repr(x)+'Y')

        # Make sure whitespace around the = works.
        self.assertEqual(f'X{x  =}Y', 'Xx  ='+repr(x)+'Y')
        self.assertEqual(f'X{x=  }Y', 'Xx=  '+repr(x)+'Y')
        self.assertEqual(f'X{x  =  }Y', 'Xx  =  '+repr(x)+'Y')
        self.assertEqual(f"sadsd {1 + 1 =  :{1 + 1:1d}f}", "sadsd 1 + 1 =  2.000000")

        # These next lines contains tabs.  Backslash escapes don't
        # work in f-strings.
        # patchcheck doesn't like these tabs.  So the only way to test
        # this will be to dynamically created and exec the f-strings.  But
        # that's such a hassle I'll save it for another day.  For now, convert
        # the tabs to spaces just to shut up patchcheck.
        #self.assertEqual(f'X{x =}Y', 'Xx\t='+repr(x)+'Y')
        #self.assertEqual(f'X{x =       }Y', 'Xx\t=\t'+repr(x)+'Y')

    def test_walrus(self):
        x = 20
        # This isn't an assignment expression, it's 'x', with a format
        # spec of '=10'.
        self.assertEqual(f'{x:=10}', '        20')

        # This is an assignment expression, which requires parens.
        self.assertEqual(f'{(x:=10)}', '10')
        self.assertEqual(x, 10)

    def test_invalid_syntax_error_message(self):
        with self.assertRaisesRegex(SyntaxError,
                                    "f-string: expecting '=', or '!', or ':', or '}'"):
            self.fragment("f'{a $ b}'")

    def test_with_two_commas_in_format_specifier(self):
        error_msg = (re.escape("Cannot specify ',' with ','.")
            if sys.version_info >= (3, 9) else "Cannot specify .*")
        with self.assertRaisesRegex(ValueError, error_msg):
            f'{1:,,}'

    def test_with_two_underscore_in_format_specifier(self):
        error_msg = re.escape("Cannot specify '_' with '_'.")
        with self.assertRaisesRegex(ValueError, error_msg):
            f'{1:__}'

    def test_with_a_commas_and_an_underscore_in_format_specifier(self):
        error_msg = re.escape("Cannot specify both ',' and '_'.")
        with self.assertRaisesRegex(ValueError, error_msg):
            f'{1:,_}'

    def test_with_an_underscore_and_a_comma_in_format_specifier(self):
        error_msg = re.escape("Cannot specify both ',' and '_'.")
        with self.assertRaisesRegex(ValueError, error_msg):
            f'{1:_,}'

    # Cython raises the error, but later in the pipeline.
    # The test is in e_fstring2 instead.
    def __test_syntax_error_for_starred_expressions(self):
        with self.assertRaisesRegex(SyntaxError, "can't use starred expression here"):
            self.fragment("f'{*a}'")

        with self.assertRaisesRegex(SyntaxError,
                                    "f-string: expecting a valid expression after '{'"):
            self.fragment("f'{**a}'")

    def test_not_closing_quotes(self):
        self.assertAllRaise(SyntaxError, "unterminated f-string literal", ['f"', "f'"])
        self.assertAllRaise(SyntaxError, "unterminated triple-quoted f-string literal",
                            ['f"""', "f'''"])
        # Ensure that the errors are reported at the correct line number.
        data = '''\
    x = 1 + 1
    y = 2 + 2
    z = f"""
    sdfjnsdfjsdf
    sdfsdfs{1+
    2} dfigdf {3+
    4}sdufsd""
    '''
        try:
            self.fragment(data)
        except (SyntaxError, CompileError) as e:
            pass  # We don't stand a chance of replicating this
            #self.assertEqual(e.text, 'z = f"""')
            #self.assertEqual(e.lineno, 3)
    def test_syntax_error_after_debug(self):
        self.assertAllRaise(SyntaxError, "f-string: expecting a valid expression after '{'",
                            [
                                "f'{1=}{;'",
                                "f'{1=}{+;'",
                                "f'{1=}{2}{;'",
                                "f'{1=}{3}{;'",
                            ])
        self.assertAllRaise(SyntaxError, "f-string: expecting '=', or '!', or ':', or '}'",
                            [
                                "f'{1=}{1;'",
                                "f'{1=}{1;}'",
                            ])

    def __test_debug_in_file(self):
        with temp_cwd():
            script = 'script.py'
            with open('script.py', 'w') as f:
                f.write(f"""\
    print(f'''{{
    3
    =}}''')""")

            _, stdout, _ = assert_python_ok(script)
        self.assertEqual(stdout.decode('utf-8').strip().replace('\r\n', '\n').replace('\r', '\n'),
                         "3\n=3")


if __name__ == '__main__':
    unittest.main()



class MySpecialException(Exception):
    def method(self):
        return "method"
    
    def __neg__(self):
        return "neg"

    def __abs__(self):
        return "abs"

    def __len__(self):
        return 100

    def __add__(self, other):
        return f"add {other}"

    # This overrides a method of BaseException
    def with_traceback(self, tb):
        return "ecart"

def test_exception_method():
    """
    >>> test_exception_method()
    'method'
    """
    try:
        raise MySpecialException()
    except MySpecialException as e:
        return e.method()

def test_exception_minus():
    """
    >>> test_exception_minus()
    'neg'
    """
    try:
        raise MySpecialException()
    except MySpecialException as e:
        return -e

def test_exception_abs():
    """
    >>> test_exception_abs()
    'abs'
    """
    try:
        raise MySpecialException()
    except MySpecialException as e:
        return abs(e)

def test_exception_len():
    """
    >>> test_exception_len()
    100
    """
    try:
        raise MySpecialException()
    except MySpecialException as e:
        return len(e)


def test_exception_sum():
    """
    >>> test_exception_sum()
    'add 1'
    """
    try:
        raise MySpecialException()
    except MySpecialException as e:
        return e+1

def test_known_method():
    """
    >>> test_known_method()
    'ecart'
    """
    try:
        raise MySpecialException()
    except MySpecialException as e:
        return e.with_traceback(None)



from distutils.core import setup
from Cython.Build import cythonize

directives = {
    'optimize.inline_defnode_calls': True
}

setup(
    name='benchmarks',
    ext_modules=cythonize("*.py", language_level=3, annotate=True,
                          compiler_directives=directives,
                          exclude=["setup.py"]),
)



# mode: run
# tag: cpp, cpp11, no-cpp-locals
# Note - keep this file as cpp11 (and put cpp20 tests in a separate file)
# to make sure that it's usable in cpp11 mode.

# cython: language_level=3

from libcpp.condition_variable cimport *
from libcpp.mutex cimport mutex, timed_mutex, unique_lock, py_safe_construct_unique_lock
from libcpp cimport bool

from threading import Thread
import time

# define some placeholder chrono stuff for the sake of testing
cdef extern from "<chrono>" namespace "std::chrono" nogil:
    cdef cppclass milliseconds:
        milliseconds()
        milliseconds(long)

cdef extern from *:
    """
    namespace {
        std::chrono::time_point<std::chrono::steady_clock> get_time_point() {
            return std::chrono::time_point<std::chrono::steady_clock>::max();
        }

        using time_point_type = decltype(get_time_point());
    }
    """
    cdef cppclass time_point_type:
        pass

    time_point_type get_time_point()

cdef int global_value = 0

cdef bool global_value_predicate() noexcept nogil:
    return global_value > 0

ctypedef bool (*cfunc_predicate)() noexcept nogil

cpdef enum:
    NO_NOTIFY
    NOTIFY_ALL
    NOTIFY_ONE

cdef class CVHelperClass:
    cdef condition_variable cv
    cdef mutex m
    cdef const char* outcome

    def notify(self, int how_to_notify, new_value=None):
        global global_value

        if how_to_notify == NO_NOTIFY:
            return

        cdef bint new_value_is_none = new_value is None
        cdef int new_value_int = new_value if not new_value_is_none else -1
        with nogil:
            self.m.lock()
            try:
                if not new_value_is_none:
                    global_value = new_value_int
            finally:
                self.m.unlock()
            if how_to_notify == NOTIFY_ALL:
                self.cv.notify_all()
            elif how_to_notify == NOTIFY_ONE:
                self.cv.notify_one()

    cdef wait_on_new_thread(self, cfunc_predicate pred):
        with nogil:
            l = unique_lock[mutex](self.m)
        def thread_func():
            with nogil:
                l2 = unique_lock[mutex](self.m)
                if pred:
                    self.cv.wait(l2, pred)
                else:
                    self.cv.wait(l2)
        t = Thread(target=thread_func)
        t.start()
        return t

    cdef wait_for_on_new_thread(self, milliseconds duration, cfunc_predicate pred):
        with nogil:
            l = unique_lock[mutex](self.m)
        def thread_func():
            with nogil:
                l2 = unique_lock[mutex](self.m)
                if pred:
                    self.outcome = ("true" if self.cv.wait_for(l2, duration, pred) else "false")
                else:
                    self.outcome = ("timeout" if self.cv.wait_for(l2, duration) == cv_status.timeout else "no timeout")
        t = Thread(target=thread_func)
        t.start()
        return t


def test_cv_wait(bint use_predicate, int how_to_notify):
    """
    >>> test_cv_wait(False, NOTIFY_ALL)
    >>> test_cv_wait(False, NOTIFY_ONE)
    >>> test_cv_wait(True, NOTIFY_ALL)
    """
    global global_value
    global_value = 0
    helper = CVHelperClass()
    t = helper.wait_on_new_thread(global_value_predicate if use_predicate else <cfunc_predicate>NULL)
    if use_predicate:
        helper.notify(how_to_notify=how_to_notify, new_value=5)
        t.join()
    else:
        while t.is_alive():
            helper.notify(how_to_notify=how_to_notify, new_value=None)
            t.join(0.0)

def test_cv_wait_for(bint use_predicate, int how_to_notify, int ms):
    """
    >>> test_cv_wait_for(False, NO_NOTIFY, 1)
    b'timeout'
    >>> test_cv_wait_for(True, NO_NOTIFY, 1)
    b'false'
    >>> test_cv_wait_for(True, NOTIFY_ALL, 10000)
    b'true'
    >>> test_cv_wait_for(False, NOTIFY_ONE, 10000)
    b'no timeout'
    """
    global global_value
    global_value = 0
    helper = CVHelperClass()
    t = helper.wait_for_on_new_thread(milliseconds(ms), global_value_predicate if use_predicate else <cfunc_predicate>NULL)
    if use_predicate:
        helper.notify(how_to_notify=how_to_notify, new_value=5)
        t.join()
    else:
        while t.is_alive():
            helper.notify(how_to_notify=how_to_notify, new_value=None)
            t.join(0.0)
    return helper.outcome


# Use condition_variable_any and a different mutex type, but otherwise the tests
# are identical to the condition_variable ones
cdef class CVAnyHelperClass:
    cdef condition_variable_any cv
    cdef timed_mutex m
    cdef const char* outcome

    def notify(self, int how_to_notify, new_value=None):
        global global_value

        if how_to_notify == NO_NOTIFY:
            return

        cdef bint new_value_is_none = new_value is None
        cdef int new_value_int = new_value if not new_value_is_none else -1
        with nogil:
            self.m.lock()
            try:
                if not new_value_is_none:
                    global_value = new_value_int
            finally:
                self.m.unlock()
            if how_to_notify == NOTIFY_ALL:
                self.cv.notify_all()
            elif how_to_notify == NOTIFY_ONE:
                self.cv.notify_one()

    cdef wait_on_new_thread(self, cfunc_predicate pred):
        with nogil:
            l = unique_lock[timed_mutex](self.m)
        def thread_func():
            with nogil:
                l2 = unique_lock[timed_mutex](self.m)
                if pred:
                    self.cv.wait(l2, pred)
                else:
                    self.cv.wait(l2)
        t = Thread(target=thread_func)
        t.start()
        return t

    cdef wait_for_on_new_thread(self, milliseconds duration, cfunc_predicate pred):
        with nogil:
            l = unique_lock[timed_mutex](self.m)
        def thread_func():
            with nogil:
                l2 = unique_lock[timed_mutex](self.m)
                if pred:
                    self.outcome = ("true" if self.cv.wait_for(l2, duration, pred) else "false")
                else:
                    self.outcome = ("timeout" if self.cv.wait_for(l2, duration) == cv_status.timeout else "no timeout")
        t = Thread(target=thread_func)
        t.start()
        return t


def test_cv_any_wait(bint use_predicate, int how_to_notify):
    """
    >>> test_cv_any_wait(False, NOTIFY_ALL)
    >>> test_cv_any_wait(False, NOTIFY_ONE)
    >>> test_cv_any_wait(True, NOTIFY_ALL)
    """
    global global_value
    global_value = 0
    helper = CVAnyHelperClass()
    t = helper.wait_on_new_thread(global_value_predicate if use_predicate else <cfunc_predicate>NULL)
    if use_predicate:
        helper.notify(how_to_notify=how_to_notify, new_value=5)
        t.join()
    else:
        while t.is_alive():
            helper.notify(how_to_notify=how_to_notify, new_value=None)
            t.join(0.0)

def test_cv_any_wait_for(bint use_predicate, int how_to_notify, int ms):
    """
    >>> test_cv_any_wait_for(False, NO_NOTIFY, 1)
    b'timeout'
    >>> test_cv_any_wait_for(True, NO_NOTIFY, 1)
    b'false'
    >>> test_cv_any_wait_for(True, NOTIFY_ALL, 10000)
    b'true'
    >>> test_cv_any_wait_for(False, NOTIFY_ONE, 10000)
    b'no timeout'
    """
    global global_value
    global_value = 0
    helper = CVAnyHelperClass()
    t = helper.wait_for_on_new_thread(milliseconds(ms), global_value_predicate if use_predicate else <cfunc_predicate>NULL)
    if use_predicate:
        helper.notify(how_to_notify=how_to_notify, new_value=5)
        t.join()
    else:
        while t.is_alive():
            helper.notify(how_to_notify=how_to_notify, new_value=None)
            t.join(0.0)
    return helper.outcome

def test_py_safe_wait_basic(sleep_for):
    """
    >>> test_py_safe_wait_basic(None)
    >>> test_py_safe_wait_basic(0.05)
    >>> test_py_safe_wait_basic(0.0)
    """
    cdef condition_variable cv
    cdef mutex m
    l = unique_lock[mutex](m)

    def trigger():
        # needs the GIL
        l2 = py_safe_construct_unique_lock(m)
        cv.notify_all()
        if sleep_for is not None:
            time.sleep(sleep_for)

    t = Thread(target=trigger)
    t.start()
    py_safe_wait(cv, l)
    t.join()

    # try wait_for (testing compilation, rather than timeout)
    t = Thread(target=trigger)
    t.start()
    py_safe_wait_for(cv, l, milliseconds(50000))
    t.join()

    # try wait_until (testing compilation, rather than timeout)
    t = Thread(target=trigger)
    t.start()
    py_safe_wait_until(cv, l, get_time_point())
    t.join()

def test_py_safe_wait_object(sleep_for):
    """
    >>> test_py_safe_wait_object(None)
    >>> test_py_safe_wait_object(0.05)
    >>> test_py_safe_wait_object(0.0)
    """
    cdef condition_variable cv
    cdef mutex m
    success = False
    l = unique_lock[mutex](m)

    import sys

    def predicate():
        # The condition variable will always call this with the lock held
        return success

    def trigger():
        nonlocal success
        # needs the GIL (for the sake of testing)
        l2 = py_safe_construct_unique_lock(m)
        success = True
        cv.notify_all()
        if sleep_for is not None:
            time.sleep(sleep_for)

    t = Thread(target=trigger)
    t.start()
    py_safe_object_wait(cv, l, predicate)
    t.join()

    # try wait_for (testing compilation, rather than timeout)
    success = False
    t = Thread(target=trigger)
    t.start()
    py_safe_object_wait_for(cv, l, milliseconds(50000), predicate)
    t.join()

    # try wait_until (testing compilation, rather than timeout)
    success = False
    t = Thread(target=trigger)
    t.start()
    py_safe_object_wait_until(cv, l, get_time_point(), predicate)
    t.join()

def test_py_safe_wait_p(sleep_for):
    """
    >>> test_py_safe_wait_p(None)
    >>> test_py_safe_wait_p(0.05)
    >>> test_py_safe_wait_p(0.0)
    """
    global global_value
    global_value = 0

    cdef condition_variable cv
    cdef mutex m
    l = unique_lock[mutex](m)

    def trigger():
        global global_value
        # needs the GIL (for the sake of testing)
        l2 = py_safe_construct_unique_lock(m)
        global_value = 1
        cv.notify_all()
        if sleep_for is not None:
            time.sleep(sleep_for)

    t = Thread(target=trigger)
    t.start()
    py_safe_wait(cv, l, global_value_predicate)
    t.join()

    # try wait_for (testing compilation, rather than timeout)
    global_value = 0
    t = Thread(target=trigger)
    t.start()
    py_safe_wait_for(cv, l, milliseconds(50000), global_value_predicate)
    t.join()

    # try wait_until (testing compilation, rather than timeout)
    global_value = 0
    t = Thread(target=trigger)
    t.start()
    py_safe_wait_until(cv, l, get_time_point(), global_value_predicate)
    t.join()



# mode: run

def return_x(x):
    return x

def for_from_in_try(x):
    """
    >>> for_from_in_try(10)
    'Except return 10'
    >>> for_from_in_try(0)
    'Normal return 0'
    """
    try:
        for 0 <= _ < (a := return_x(x)):
            raise RuntimeError
        
        return f'Normal return {a}'
    except Exception:
        return f'Except return {a}'



# mode: run

import cython


def calls():
    """
    >>> calls()
    """
    r1 = range(3)
    assert cython.typeof(r1) == 'range object', cython.typeof(r1)

    r2 = range(1, 3)
    assert cython.typeof(r2) == 'range object', cython.typeof(r2)

    r3 = range(1, 3, 1)
    assert cython.typeof(r3) == 'range object', cython.typeof(r3)


def arg_type(r: range):
    """
    >>> arg_type(range(3))
    0
    1
    2
    >>> arg_type([])
    Traceback (most recent call last):
    TypeError: Argument 'r' has incorrect type (expected range, got list)
    """
    for i in r:
        print(i)


def attributes(r: range):
    """
    >>> attributes(range(2))
    (0, 2, 1)
    >>> attributes(range(1,2))
    (1, 2, 1)
    >>> attributes(range(1,2,3))
    (1, 2, 3)
    """
    return r.start, r.stop, r.step


def isinstance_range(x):
    """
    >>> isinstance_range(range(20))
    True
    >>> isinstance_range([])
    False
    >>> isinstance_range(2)
    False
    """
    return isinstance(x, range)



# mode: error

# This is split into many sub-files because some errors terminate the parser

t'{x!}'
t'{x=!}'
t'{x!z}'        
t'{lambda:1}'       
t'{x:{;}}'
t'{1:d\n}'

_ERRORS = """
5:5: missing conversion character
6:6: missing conversion character
7:5: invalid conversion character 'z'
# TODO This message could probably be improved
8:9: Expected an identifier, found 'EOF'
8:9: Unrecognized character
"""



from distutils.core import setup
from distutils.extension import Extension
from Cython.Build import cythonize

setup(
    name = 'callback',
    ext_modules=cythonize([
        Extension("cheese", ["cheese.pyx", "cheesefinder.c"]),
    ]),
)



import cython

import array
import collections
import time


def _unpack_buffer_const_char_1d(provider, int number, timer=time.perf_counter):
    cdef const unsigned char[:] buffer

    t = timer()
    for _ in range(number):
        buffer = provider
    t = timer() - t
    return t


def bm_unpack_buffer_const_char_1d(number, timer=time.perf_counter):
    data = bytes(1000)
    #cdef const unsigned char[:] memview = data
    return {
        'unpack_buffer[const char 1d, bytes]': _unpack_buffer_const_char_1d(data, number, timer),
        'unpack_buffer[const char 1d, bytearray]': _unpack_buffer_const_char_1d(bytearray(data), number, timer),
        'unpack_buffer[const char 1d, array]': _unpack_buffer_const_char_1d(array.array('B', data), number, timer),
        #'unpack_buffer[const char 1d, cymemview]': _unpack_buffer_const_char_1d(memview, number, timer),
    }


def _with_contextmanager_pass(cm, int number, timer=time.perf_counter):
    t = timer()
    for _ in range(number):
        with cm:
            pass
    t = timer() - t
    return t


def _with_contextmanager_raise(cm, int number, timer=time.perf_counter):
    exception = TypeError()
    t = timer()
    for _ in range(number):
        with cm:
            raise exception
    t = timer() - t
    return t


class PyCM:
    def __enter__(self): pass
    def __exit__(self, ex1, ex2, ex3): return True


cdef class CyCM:
    def __enter__(self): pass
    def __exit__(self, ex1, ex2, ex3): return True


def bm_with_statement(number, timer=time.perf_counter):
    return {
        'with_pass_PyCM': _with_contextmanager_pass(PyCM(), number, timer),
        'with_pass_CyCM': _with_contextmanager_pass(CyCM(), number, timer),
        'with_raise_PyCM': _with_contextmanager_raise(PyCM(), number, timer),
        'with_raise_CyCM': _with_contextmanager_raise(CyCM(), number, timer),
    }


def bm_create_inner_function(int number, timer=time.perf_counter):
    t = timer()
    for _ in range(number):
        def inner_a(arg1, int arg2):
            pass
        def inner_b(arg1, int arg2):
            pass
        def inner_c(arg1, int arg2):
            pass
    plain_time = timer() - t

    t = timer()
    for _ in range(number):
        def inner1():
            pass
        def inner2(arg1, int arg2):
            return inner1()
        def inner3(arg1, arg2=inner1):
            return inner2()
    closure_time = timer() - t

    return {
        'create_inner_function[plain]': plain_time,
        'create_inner_function[closure]': closure_time,
    }


def bm_iter_string_literal(int number, timer=time.perf_counter):
    # iterate over first digits of π
    any_none: bool

    # str
    t = timer()
    for _ in range(number):
        [ch for ch in (
            "3141592653589793238462643383279502884197169399375105820974944592307816406286"  # 76 characters
            "2089986280348253421170679821480865132823066470938446095505822317253594081284"  # 76 characters
        )]

    any_none = False
    for _ in range(number):
        for ch in (
                "3141592653589793238462643383279502884197169399375105820974944592307816406286"  # 76 characters
                "2089986280348253421170679821480865132823066470938446095505822317253594081284"  # 76 characters
                ):
            any_none |= (ch is None)
    str_time = timer() - t
    assert not any_none

    # bytes
    t = timer()
    for _ in range(number):
        [ch for ch in (
            b"3141592653589793238462643383279502884197169399375105820974944592307816406286"  # 76 characters
            b"2089986280348253421170679821480865132823066470938446095505822317253594081284"  # 76 characters
        )]

    any_none = False
    for _ in range(number):
        for ch in (
                b"3141592653589793238462643383279502884197169399375105820974944592307816406286"  # 76 characters
                b"2089986280348253421170679821480865132823066470938446095505822317253594081284"  # 76 characters
                ):
            any_none |= (ch is None)
    bytes_time = timer() - t
    assert not any_none

    return {
        'iter_str': str_time,
        'iter_bytes': bytes_time,
    }


def run_benchmark(repeat: cython.int = 10, number=100, timer=time.perf_counter):
    i: cython.int

    collected_timings = collections.defaultdict(list)

    for name, func in globals().items():
        if not name.startswith('bm_'):
            continue

        for i in range(repeat):
            timings = func(number, timer)
            for name, t in timings.items():
                collected_timings[name].append(t)

    for name, timings in collected_timings.items():
        print(f"{name}: {timings}")



import sys
import re

POS_MATCH = re.compile(r"^[^:]+:\d+:\d+: ")

def check_file(filename) -> int:
    failed_count = 0
    with open(filename) as f:
        for line in f:
            if not POS_MATCH.match(line) or line.startswith("WARNING: ThreadSanitizer"):
                continue
            if line.startswith("conftest.c"):
                # this is in the Python setup - we don't care
                continue
            if "applying zero offset to null pointer" in line:
                # This is OK in C++ and dropped in clang21 (on DW's laptop) so treat it as fine.
                continue
            # anything not specifically included is a failure
            failed_count += 1
            print(line)
    return failed_count

failed_count = 0

if len(sys.argv) == 2 and sys.argv[1].endswith('.*'):
    # No issues so the pattern has not expanded
    print(f"No logs found with pattern '{sys.argv[1]}'")
    exit(0)
for arg in sys.argv[1:]:
    print(f"Looking at file '{arg}':")
    failed_count += check_file(arg)

exit(failed_count)



# mode: error
# coding: ASCII
# tag: cpp, cpp20

from libcpp.string_view cimport string_view

def get_obj():
    return b"123"

cdef bytes get_bytes():
    return b"123"

cdef string_view s1 = get_obj()  # bad, reference to temp
cdef string_view s2 = get_bytes()  # bad, reference to temp
cdef string_view s3 = u"123".encode()  # bad, reference to temp

_ERRORS = """
13:0: Storing unsafe C derivative of temporary Python reference
14:0: Storing unsafe C derivative of temporary Python reference
15:0: Storing unsafe C derivative of temporary Python reference
"""



import os
import tempfile
import unittest
from Cython.Shadow import inline
from Cython.Build.Inline import safe_type, cymeit
from Cython.TestUtils import CythonTest

try:
    import numpy
    has_numpy = True
except:
    has_numpy = False

test_kwds = dict(force=True, quiet=True)

global_value = 100

class TestInline(CythonTest):
    def setUp(self):
        CythonTest.setUp(self)
        self._call_kwds = dict(test_kwds)
        if os.path.isdir('TEST_TMP'):
            lib_dir = os.path.join('TEST_TMP','inline')
        else:
            lib_dir = tempfile.mkdtemp(prefix='cython_inline_')
        self._call_kwds['lib_dir'] = lib_dir

    def test_simple(self):
        self.assertEqual(inline("return 1+2", **self._call_kwds), 3)

    def test_types(self):
        self.assertEqual(inline("""
            cimport cython
            return cython.typeof(a), cython.typeof(b)
        """, a=1.0, b=[], **self._call_kwds), ('double', 'list object'))

    def test_locals(self):
        a = 1
        b = 2
        self.assertEqual(inline("return a+b", **self._call_kwds), 3)

    def test_globals(self):
        self.assertEqual(inline("return global_value + 1", **self._call_kwds), global_value + 1)

    def test_no_return(self):
        self.assertEqual(inline("""
            a = 1
            cdef double b = 2
            cdef c = []
        """, **self._call_kwds), dict(a=1, b=2.0, c=[]))

    def test_def_node(self):
        foo = inline("def foo(x): return x * x", **self._call_kwds)['foo']
        self.assertEqual(foo(7), 49)

    def test_class_ref(self):
        class Type:
            pass
        tp = inline("Type")['Type']
        self.assertEqual(tp, Type)

    def test_pure(self):
        import cython as cy
        b = inline("""
        b = cy.declare(float, a)
        c = cy.declare(cy.pointer(cy.float), &b)
        return b
        """, a=3, **self._call_kwds)
        self.assertEqual(type(b), float)

    def test_compiler_directives(self):
        self.assertEqual(
            inline('return sum(x)',
                   x=[1, 2, 3],
                   cython_compiler_directives={'boundscheck': False}),
            6
        )

    def test_lang_version(self):
        # GH-3419. Caching for inline code didn't always respect compiler directives.
        inline_divcode = "def f(int a, int b): return a/b"
        self.assertEqual(
            inline(inline_divcode, language_level=2)['f'](5,2),
            2
        )
        self.assertEqual(
            inline(inline_divcode, language_level=3)['f'](5,2),
            2.5
        )
        self.assertEqual(
            inline(inline_divcode, language_level=2)['f'](5,2),
            2
        )

    def test_repeated_use(self):
        inline_mulcode = "def f(int a, int b): return a * b"
        self.assertEqual(inline(inline_mulcode)['f'](5, 2), 10)
        self.assertEqual(inline(inline_mulcode)['f'](5, 3), 15)
        self.assertEqual(inline(inline_mulcode)['f'](6, 2), 12)
        self.assertEqual(inline(inline_mulcode)['f'](5, 2), 10)

        f = inline(inline_mulcode)['f']
        self.assertEqual(f(5, 2), 10)
        self.assertEqual(f(5, 3), 15)

    @unittest.skipIf(not has_numpy, "NumPy is not available")
    def test_numpy(self):
        import numpy
        a = numpy.ndarray((10, 20))
        a[0,0] = 10
        self.assertEqual(safe_type(a), 'numpy.ndarray[numpy.float64_t, ndim=2]')
        self.assertEqual(inline("return a[0,0]", a=a, **self._call_kwds), 10.0)


class TestCymeit(unittest.TestCase):
    def _run(self, code, setup_code=None, **kwargs):
        timings, number = cymeit(code, setup_code=setup_code, **kwargs)

        self.assertGreater(min(timings), 0)

        # Guard that autoscaling leads to reasonable timings.
        # Note: we cannot compare against the expected 0.2 due to large timing variations on CI.
        max_time = max(timing * number for timing in timings)
        if isinstance(max_time, int):
            self.assertGreaterEqual(max_time, 100_000)
        else:
            self.assertGreaterEqual(max_time, 0.0001)
        self.assertGreater(number, 4)  # arbitrary lower bound for our very quick benchmarks

        return timings

    def test_benchmark_simple(self):
        setup_code = "numbers = list(range(0, 1000, 3))"
        self._run("sum([num for num in numbers])", setup_code, repeat=3)

    def test_benchmark_timer(self):
        import time
        setup_code = "numbers = list(range(0, 1000, 3))"
        timings = self._run("sum([num for num in numbers])", setup_code, timer=time.perf_counter, repeat=3)

        for timing in timings:
            self.assertIsInstance(timing, float)

    def test_benchmark_timer_ns(self):
        import time
        setup_code = "numbers = list(range(0, 1000, 3))"
        timings = self._run("sum([num for num in numbers])", setup_code, timer=time.perf_counter_ns, repeat=3)

        for timing in timings:
            self.assertIsInstance(timing, int)

    def test_benchmark_multiline_setup(self):
        setup_code = """
        numbers = list(range(0, 100, 3))

        def csum(numbers):
            result = 0
            for number in numbers:
                result += number
            return result
        """
        self._run("csum(numbers)", setup_code)

    def test_benchmark_multiline_code(self):
        setup_code = "numbers = list(range(0, 100, 3))"
        self._run("""
            sum([
                num
                    for num in numbers
                ])
            """,
            setup_code,
            repeat=3
        )

    def test_benchmark_in_module(self):
        self._run("fsum(range(100))", import_module='math', repeat=2)



# mode: run

from __future__ import with_statement

def typename(t):
    name = type(t).__name__
    return u"<type '%s'>" % name

class MyException(Exception):
    pass

class ContextManager(object):
    def __init__(self, value, exit_ret = None):
        self.value = value
        self.exit_ret = exit_ret

    def __exit__(self, a, b, tb):
        print u"exit", typename(a), typename(b), typename(tb)
        return self.exit_ret

    def __enter__(self):
        print u"enter"
        return self.value

def no_as():
    """
    >>> no_as()
    enter
    hello
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    """
    with ContextManager(u"value"):
        print u"hello"

def basic():
    """
    >>> basic()
    enter
    value
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    """
    with ContextManager(u"value") as x:
        print x

def with_pass():
    """
    >>> with_pass()
    enter
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    """
    with ContextManager(u"value") as x:
        pass

def with_exception(exit_ret):
    """
    >>> with_exception(None)
    enter
    value
    exit <type 'type'> <type 'MyException'> <type 'traceback'>
    outer except
    >>> with_exception(True)
    enter
    value
    exit <type 'type'> <type 'MyException'> <type 'traceback'>
    """
    try:
        with ContextManager(u"value", exit_ret=exit_ret) as value:
            print value
            raise MyException()
    except:
        print u"outer except"

def multitarget():
    """
    >>> multitarget()
    enter
    1 2 3 4 5
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    """
    with ContextManager((1, 2, (3, (4, 5)))) as (a, b, (c, (d, e))):
        print a, b, c, d, e

def tupletarget():
    """
    >>> tupletarget()
    enter
    (1, 2, (3, (4, 5)))
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    """
    with ContextManager((1, 2, (3, (4, 5)))) as t:
        print t

def typed():
    """
    >>> typed()
    enter
    10
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    """
    cdef unsigned char i
    c = ContextManager(255)
    with c as i:
        i += 11
        print i

def multimanager():
    """
    >>> multimanager()
    enter
    enter
    enter
    enter
    enter
    enter
    2
    value
    1 2 3 4 5
    nested
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    """
    with ContextManager(1), ContextManager(2) as x, ContextManager(u'value') as y,\
            ContextManager(3), ContextManager((1, 2, (3, (4, 5)))) as (a, b, (c, (d, e))):
        with ContextManager(u'nested') as nested:
            print x
            print y
            print a, b, c, d, e
            print nested

# Tests borrowed from pyregr test_with.py,
# modified to follow the constraints of Cython.
import unittest

class Dummy(object):
    def __init__(self, value=None, gobble=False):
        if value is None:
            value = self
        self.value = value
        self.gobble = gobble
        self.enter_called = False
        self.exit_called = False

    def __enter__(self):
        self.enter_called = True
        return self.value

    def __exit__(self, *exc_info):
        self.exit_called = True
        self.exc_info = exc_info
        if self.gobble:
            return True

class InitRaises(object):
    def __init__(self): raise RuntimeError()

class EnterRaises(object):
    def __enter__(self): raise RuntimeError()
    def __exit__(self, *exc_info): pass

class ExitRaises(object):
    def __enter__(self): pass
    def __exit__(self, *exc_info): raise RuntimeError()

class NestedWith(unittest.TestCase):
    """
    >>> NestedWith().runTest()
    """

    def runTest(self):
        self.testNoExceptions()
        self.testExceptionInExprList()
        self.testExceptionInEnter()
        self.testExceptionInExit()
        self.testEnterReturnsTuple()

    def testNoExceptions(self):
        with Dummy() as a, Dummy() as b:
            self.assertTrue(a.enter_called)
            self.assertTrue(b.enter_called)
        self.assertTrue(a.exit_called)
        self.assertTrue(b.exit_called)

    def testExceptionInExprList(self):
        try:
            with Dummy() as a, InitRaises():
                pass
        except:
            pass
        self.assertTrue(a.enter_called)
        self.assertTrue(a.exit_called)

    def testExceptionInEnter(self):
        try:
            with Dummy() as a, EnterRaises():
                self.fail('body of bad with executed')
        except RuntimeError:
            pass
        else:
            self.fail('RuntimeError not reraised')
        self.assertTrue(a.enter_called)
        self.assertTrue(a.exit_called)

    def testExceptionInExit(self):
        body_executed = False
        with Dummy(gobble=True) as a, ExitRaises():
            body_executed = True
        self.assertTrue(a.enter_called)
        self.assertTrue(a.exit_called)
        self.assertTrue(body_executed)
        self.assertNotEqual(a.exc_info[0], None)

    def testEnterReturnsTuple(self):
        with Dummy(value=(1,2)) as (a1, a2), \
             Dummy(value=(10, 20)) as (b1, b2):
            self.assertEqual(1, a1)
            self.assertEqual(2, a2)
            self.assertEqual(10, b1)
            self.assertEqual(20, b2)


def long_with_block(a):
    """
    https://github.com/cython/cython/issues/7122 - copying the with block
    used to generate recursion error
    >>> long_with_block(0)
    enter
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    336
    """
    with ContextManager(None):
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
        a += 1
    print(a)

ctypedef fused numeric:
    int
    float

def with_fused(numeric x, bint raise_something):
    """
    Both fused types and with block involve awkward deepcopies and
    so combining them should be checked.

    >>> with_fused(1, True)
    Traceback (most recent call last):
        ...
    RuntimeError
    >>> with_fused(1, False)
    enter
    1
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    >>> with_fused(2.5, True)
    Traceback (most recent call last):
        ...
    RuntimeError
    >>> with_fused(2.5, False)
    enter
    2.5
    exit <type 'NoneType'> <type 'NoneType'> <type 'NoneType'>
    """
    with ContextManager(None):
        if raise_something:
            raise RuntimeError()
        else:
            print(x)



# mode: run
# tag: openmp

cimport cython
cimport cython.parallel

from threading import Thread

cdef cython.pymutex global_lock

cdef void hide_the_reduction(int *x) noexcept nogil:
    x[0] = x[0] + 1

def global_lock_with_prange():
    """
    >>> global_lock_with_prange()
    5000
    """
    cdef int count = 0
    cdef int i
    for i in cython.parallel.prange(5000, nogil=True):
        with global_lock:
            hide_the_reduction(&count)
    return count

def local_lock_with_prange():
    """
    >>> local_lock_with_prange()
    5000
    """
    cdef int count = 0
    cdef int i
    cdef cython.pymutex lock
    for i in cython.parallel.prange(5000, nogil=True):
        with lock:
            hide_the_reduction(&count)
    return count

cdef class HasLockAttribute:
    cdef cython.pymutex lock

def lock_on_attribute(HasLockAttribute has_lock):
    """
    >>> lock_on_attribute(HasLockAttribute())
    5000
    """
    cdef int count = 0
    cdef int i
    for i in cython.parallel.prange(5000, nogil=True):
        with has_lock.lock:
            hide_the_reduction(&count)
    return count

def lock_in_closure():
    """
    >>> lock_in_closure()
    5000
    """
    cdef int count = 0
    cdef int i
    # pythread_type_lock and pymutex should behave basically the same. So test them both
    cdef cython.pythread_type_lock lock

    def inner():
        with lock:
            hide_the_reduction(&count)

    for i in cython.parallel.prange(5000, nogil=True):
        with gil:
            inner()

    return count 

def manual_acquire_release():
    """
    >>> manual_acquire_release()
    5000
    """
    cdef int count = 0
    cdef int i
    cdef cython.pymutex lock

    for i in cython.parallel.prange(5000, nogil=True):
        # Test it both with and without the GIL
        if i % 2 == 0:
            with gil:
                lock.acquire()
                hide_the_reduction(&count)
                lock.release()
        else:
            lock.acquire()
            hide_the_reduction(&count)
            lock.release()
    return count

# Although forbidden to pass a copy of the lock, pointers are fine
cdef void acquire_and_hide(cython.pymutex* l, int* i) nogil noexcept:
    with l[0]:
        hide_the_reduction(i)

def pass_as_pointer():
    """
    >>> pass_as_pointer()
    5000
    """
    cdef int count = 0
    cdef int i
    cdef cython.pymutex lock
    for i in cython.parallel.prange(5000, nogil=True):
        acquire_and_hide(&lock, &count)
    return count


# Using C is just a quick way to inject a few Python calls
# into the generator tests.  With generators and critical sections
# putting these calls between yields would definitely break.
# We don't expect the same to apply to pymutex but it makes the test
# a little more robust.
g = {}
exec(
    """
class C:
    x = 1
    def f(self):
        self.x += 1
    """, g, g)
C = g['C']
del g

def generator_local():
    """
    Really simple generator - a bit pointless because
    nothing outside it can access the lock, but it should still work.

    >>> g = generator_local()
    >>> next(g)
    1
    >>> C().f()  # inject Python "work"
    >>> next(g)
    2
    >>> try: next(g)
    ... except StopIteration: pass
    ... else: print("Fail")
    """
    cdef cython.pymutex m
    with m:
        yield 1
        yield 2

def generator_global(value_in):
    """
    >>> g = generator_global(1)
    >>> thread_func_result = None
    >>> def thread_func():
    ...    global thread_func_result
    ...    g2 = generator_global(2)
    ...    thread_func_result = [ x for x in g2 ]
    >>> next(g)
    1
    >>> from threading import Thread; t = Thread(target=thread_func); t.start()
    >>> assert thread_func_result is None  # blocked by lock
    >>> try: next(g)
    ... except StopIteration: pass
    ... else: print("Fail")
    >>> t.join(10.)
    >>> thread_func_result
    [2]
    """
    with global_lock:
        yield value_in

def make_captured_mutex_generator():
    """
    >>> gen_func = make_captured_mutex_generator()
    >>> g = gen_func('a')
    >>> thread_func_result = None
    >>> def thread_func():
    ...    global thread_func_result
    ...    thread_func_result = [ x for x in gen_func('b') ]
    >>> next(g)
    'a 0'
    >>> from threading import Thread; t = Thread(target=thread_func); t.start()
    >>> assert thread_func_result is None  # blocked by lock
    >>> C().f()  # dummy Python work
    >>> next(g)
    'a 1'
    >>> assert thread_func_result is None  # blocked by lock
    >>> try: next(g)
    ... except StopIteration: pass
    ... else: print("Fail")
    >>> t.join(10.)
    >>> thread_func_result
    ['b 0', 'b 1']
    """
    cdef cython.pymutex m
    def g(id):
        with m:
            yield f"{id} 0"
            yield f"{id} 1"
    return g

def generator_cclass(HasLockAttribute o, id):
    """
    >>> l = HasLockAttribute()
    >>> g1 = generator_cclass(l, 'a')
    >>> thread_func_result = None
    >>> def thread_func():
    ...    global thread_func_result
    ...    thread_func_result = [ x for x in generator_cclass(l, 'b') ]
    >>> next(g1)
    'a 0'
    >>> from threading import Thread; t = Thread(target=thread_func); t.start()
    >>> assert thread_func_result is None  # blocked by lock
    >>> C().f()  # dummy Python work
    >>> next(g1)
    'a 1'
    >>> assert thread_func_result is None  # blocked by lock
    >>> try: next(g1)
    ... except StopIteration: pass
    ... else: print("Fail")
    >>> t.join(10.)
    >>> thread_func_result
    ['b 0', 'b 1']
    """
    with o.lock:
        yield f"{id} 0"
        yield f"{id} 1"

def pymutex_with_gil():
    """
    >>> pymutex_with_gil()
    4000
    """
    cdef cython.pymutex lock
    cdef int count = 0
    def thread_func():
        nonlocal count
        for i in range(1000):
            with lock:
                count += 1
    threads = [ Thread(target=thread_func) for _ in range(4) ]
    for t in threads:
        t.start()
    for t in threads:
        t.join()
    return count

def old_lock_with_gil():
    """
    >>> old_lock_with_gil()
    4000
    """
    cdef cython.pythread_type_lock lock
    cdef int count = 0
    def thread_func():
        nonlocal count
        for i in range(1000):
            with lock:
                count += 1
    threads = [ Thread(target=thread_func) for _ in range(4) ]
    for t in threads:
        t.start()
    for t in threads:
        t.join()
    return count

def pymutex_without_gil():
    """
    >>> pymutex_without_gil()
    4000
    """
    cdef cython.pymutex lock
    cdef int count = 0
    def thread_func():
        nonlocal count
        with nogil:
            for i in range(1000):
                with lock:
                    count += 1
    threads = [ Thread(target=thread_func) for _ in range(4) ]
    for t in threads:
        t.start()
    for t in threads:
        t.join()
    return count

def old_lock_without_gil():
    """
    >>> old_lock_without_gil()
    4000
    """
    cdef cython.pythread_type_lock lock
    cdef int count = 0
    def thread_func():
        nonlocal count
        with nogil:
            for i in range(1000):
                with lock:
                    count += 1
    threads = [ Thread(target=thread_func) for _ in range(4) ]
    for t in threads:
        t.start()
    for t in threads:
        t.join()
    return count

cdef void pymutex_unknown_gil_impl(cython.pymutex* lock, int* count) nogil noexcept:
    with lock[0]:
        count[0] += 1

def pymutex_unknown_gil():
    """
    >>> pymutex_unknown_gil()
    4000
    """
    cdef cython.pymutex lock
    cdef int count = 0
    def thread_func():
        for i in range(1000):
            if i % 2:
                pymutex_unknown_gil_impl(&lock, &count)
            else:
                with nogil:
                    pymutex_unknown_gil_impl(&lock, &count)
    threads = [ Thread(target=thread_func) for _ in range(4) ]
    for t in threads:
        t.start()
    for t in threads:
        t.join()
    return count

cdef void old_lock_unknown_gil_impl(cython.pythread_type_lock* lock, int* count) nogil noexcept:
    with lock[0]:
        count[0] += 1

def old_lock_unknown_gil():
    """
    >>> old_lock_unknown_gil()
    4000
    """
    cdef cython.pythread_type_lock lock
    cdef int count = 0
    def thread_func():
        for i in range(1000):
            if i % 2:
                old_lock_unknown_gil_impl(&lock, &count)
            else:
                with nogil:
                    old_lock_unknown_gil_impl(&lock, &count)
    threads = [ Thread(target=thread_func) for _ in range(4) ]
    for t in threads:
        t.start()
    for t in threads:
        t.join()
    return count
